import socketserver


class TCPRequestHandler(socketserver.BaseRequestHandler):

    def handle(self):
        ## self.request is the TCP socket connected to the client
        #self.data = self.request.recv(1024).strip()
        #print("{} wrote:".format(self.client_address[0]))
        #print(self.data)
        ## just send back the same data, but upper-cased
        #self.request.sendall(self.data.upper())
        msg = self.request.recv(1024)
        print (msg.decode("utf-8"))
        self.request.sendall("UALTGER")

if __name__ == "__main__":
    HOST, PORT = "localhost", 9999

    # Create the server, binding to localhost on port 9999
    with socketserver.TCPServer((HOST, PORT), TCPRequestHandler) as server:
        # Activate the server; this will keep running until you
        # interrupt the program with Ctrl-C
        print("I am listening... port " + str(PORT))
        server.serve_forever()

    


#!/usr/bin/env python3

import socketserver
import http.server
import logging
import cgi
import signal
import sys, os, ast
import json
import configparser
from os.path import expanduser

LIST_VPC    = "listVpc"
LIST_SUBNET = "listSubnet"
LIST_EC2    = "listEc2"

class HttpServerServicesMock:

    def listVpc(self):
        response = self.buildAwsResponse(LIST_VPC)
        response["aws"]["vpc"] = []
        response["aws"]["vpc"].append({"vpcId":"vpc-123456789"})
        response["aws"]["vpc"].append({"vpcId":"vpc-01457"})
        response["aws"]["vpc"].append({"vpcId":"vpc-789456"})
        json_str = json.dumps(response)
        return json_str

    def listEc2(self, request_json):
        response = self.buildAwsResponse(LIST_EC2)
        params   = self.buildParams(request_json)
        response["aws"]["ec2"] = {}
        response["aws"]["ec2"]["vpcId"]     = params["vpcId"]
        response["aws"]["ec2"]["subnetId"]  = params["subnetId"]
        response["aws"]["ec2"]["instances"] = []
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-123456789","privateIp":"10.0.22.2","sg":"123"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-000100010","privateIp":"10.230.233.1","sg":"456"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5ewq25552jhg","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5q5525552ads1","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5w25552221","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5qwe5fds55ew2221","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5ew525552ew221","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5fds255522ewq21","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-55525552w21","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5fdsytryt25552f1","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5fdsa5552we21","privateIp":"10.1.22.2","sg":"789"})
        response["aws"]["ec2"]["instances"].append({"instanceId":"i-5ewq5525gfdgf21","privateIp":"10.1.22.2","sg":"789"})
        json_str = json.dumps(response)
        return json_str    

    def listSubnet(self, request_json):
        response = self.buildAwsResponse(LIST_SUBNET)
        params   = self.buildParams(request_json)
        response["aws"]["vpc"] = {}
        response["aws"]["vpc"]["vpcId"] = params["vpcId"]
        response["aws"]["vpc"]["subnets"] = []
        if params["vpcId"] == "vpc-01457":
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-AAAA","scope":"public"})
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-BBBB","scope":"private"})
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-CCCC","scope":"public"})
        elif params["vpcId"] == "vpc-789456": 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D1","scope":"public"})
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D2","scope":"public"})
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D3","scope":"public"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D4","scope":"private"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D5","scope":"private"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D6","scope":"public"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-D7","scope":"private"})
        elif params["vpcId"] == "vpc-123456789":
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-1", "scope":"public"})
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-2", "scope":"public"})
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-3", "scope":"public"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-4", "scope":"public"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-5", "scope":"private"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-6", "scope":"private"}) 
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-7", "scope":"private"})    
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-8", "scope":"private"})    
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-9", "scope":"public"})    
           response["aws"]["vpc"]["subnets"].append({"subnetId":"subnetId-x1x-10","scope":"public"})    
        json_str = json.dumps(response)
        return json_str

    def buildAwsResponse(self, action):
        response = {}
        response["aws"] = {}
        response["aws"]["status"] = "OK"
        response["aws"]["action"] = action
        return response
    def buildParams(self, json):
        params = {}
        for param in json["parameters"]:
           params[param["name"]] = {}
           params[param["name"]] = param["value"]
        return params   



#!/usr/bin/env python3

import boto3
import logging
import sys
import json
import os, re
import random
from botocore.exceptions import ClientError
from datetime import datetime
from pyawscp.Functions import Functions
from pyawscp.Utils import Utils, Style
from pyawscp.PrettyTable import PrettyTable
from pyawscp.Emoticons import Emoticons
from pyawscp.Config import Config
from pyawscp.CsvDrawIo import CsvDrawIo
from pyawscp.CsvDrawIoNetworking import CsvDrawIoNetworking
from pyawscp.TableArgs import TableArgs
from pygments import highlight, lexers, formatters

LOG = logging.getLogger("app." + __name__)
LOG.setLevel(logging.INFO)

class PyEc2CP:

    def __init__(self, config):
        self.config = config
        self.tempCredentials = None

    # A main function called from user
    # Check if a specific subnetId is Public looking up its Route Table for a IGW-* 0.0.0.0/0
    # arguments:
    #      subnetId: mandatory
    def subnetIsPublic(self):
        ec2      = self.botoSession().client('ec2')
        subnetId = self.config.commandArguments

        if not subnetId:
           resultTxt = "Where is the SubnetId? You didn't tell me which one " + Emoticons.ops() +  Style.RESET
           return Utils.formatResult(Functions.SUBNET_IS_PUBLIC,resultTxt, self.config, "", True)

        filters=[{'Name': 'association.subnet-id','Values': [subnetId]}, 
                 {'Name': 'route.destination-cidr-block','Values': ['0.0.0.0/0']},
                 {'Name': 'route.gateway-id','Values': ['igw-*']}]
        if self.config.awsTagsToFilter():
           for tag in self.config.awsTags: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]}
               ) 
        route_tables = ec2.describe_route_tables(Filters=filters)

        jsonResult = ""
        if self.config.printResults: 
           jsonResult = Utils.dictToJson(route_tables)

        routeTableIgw = route_tables['RouteTables']
        if len(routeTableIgw) >= 1:
           resultTxt = "Subnet " + Style.GREEN + subnetId + Style.RESET + " is " + Style.GREEN + "PUBLIC" +  Style.RESET + "\n"

           printSeparator = False
           for route in routeTableIgw[0]["Routes"]:
               if route["DestinationCidrBlock"] == '0.0.0.0/0':
                  if route["State"]:
                     if not printSeparator:
                        resultTxt = resultTxt + Utils.separator()
                        printSeparator = True
                     resultTxt = resultTxt + "\n"
                     resultTxt = resultTxt + ' '.ljust(2,' ') + "- State...........: " + Style.GREEN + route['State'] + Style.RESET 
                  if route["GatewayId"]:
                     if not printSeparator:
                        resultTxt = resultTxt + Utils.separator()
                        printSeparator = True
                     resultTxt = resultTxt + "\n"
                     resultTxt = resultTxt + ' '.ljust(2,' ') + "- GatewayId.......: " + Style.GREEN + route['GatewayId'] + Style.RESET    
                  if routeTableIgw[0]["VpcId"]:
                     if not printSeparator:
                        resultTxt = resultTxt + Utils.separator()
                        printSeparator = True
                     resultTxt = resultTxt + "\n"
                     resultTxt = resultTxt + ' '.ljust(2,' ') + "- Vpc Id..........: " + Style.GREEN + routeTableIgw[0]['VpcId'] + Style.RESET   
           return Utils.formatResult(Functions.FUNCTIONS[Functions.SUBNET_IS_PUBLIC]["name"],resultTxt, self.config, jsonResult, True, None)
        else:
           resultTxt = ""
           subnet = ec2.describe_subnets(Filters=[{'Name':'subnet-id','Values':[subnetId]}])
           if len(subnet['Subnets']) == 0:
               resultTxt = "Subnet " + Style.GREEN + subnetId + Style.RED + " not found! " + Emoticons.ops() +  Style.RESET
           else:   
               if self.config.awsTagsToFilter():
                  resultTxt = "Subnet " + Style.GREEN + subnetId + Style.RESET + " is NOT " + Style.GREEN + "PUBLIC" +  Style.RESET + ", or either did not pass the Filters Tag! " + + Emoticons.thumbsUp()
               else:   
                  resultTxt = "Subnet " + Style.GREEN + subnetId + Style.RESET + " is NOT " + Style.GREEN + "PUBLIC" +  Style.RESET + " " + Emoticons.thumbsUp() 
           return Utils.formatResult(Functions.FUNCTIONS[Functions.SUBNET_IS_PUBLIC]["name"],resultTxt, self.config, "", True, None)     
    
    # Reusable Function, used by: 
    # -lisSubtnetsVpc
    def _checkSubnetIsPublic(self, subnetId, extraFilters):
        ec2 = self.botoSession().client('ec2')
        filters=[{'Name': 'association.subnet-id','Values': [subnetId]}, 
                 {'Name': 'route.destination-cidr-block','Values': ['0.0.0.0/0']},
                 {'Name': 'route.gateway-id','Values': ['igw-*']}
        ]
        route_tables = ec2.describe_route_tables(Filters=filters)
        routeTableIgw = route_tables['RouteTables']
        if len(routeTableIgw) >= 1:
            return True
        else:
            return False    

    # List all Subnets from a VPC
    def listSubnetsVpc(self):
        ec2   = self.botoSession().client('ec2')

        vpcId     = ""
        isPublic  = False
        tableArgs = TableArgs()
        tableArgs.setArguments(self.config.commandArguments) 

        if "," in self.config.commandArguments:
           vpcId = self.config.commandArguments.split(",")[0]
           tableArgs.setArguments(self.config.commandArguments)
           if "ispublic" in self.config.commandArguments:
               isPublic = True
        else:
           vpcId = tableArgs.cleanPipelineArguments()

        if not vpcId:
           resultTxt = "Where is the VpcId? You didn't tell me which VPC " + Emoticons.ops() +  Style.RESET
           return Utils.formatResult(Functions.LIST_SUBNETS_VPC,resultTxt, self.config, "", True, tableArgs)

        filters=[
            {
                'Name': 'vpc-id',
                'Values': [
                    vpcId
                ]
            }
        ]
        if self.config.awsTagsToFilter():
           # Environment Tags set
           for tag in self.config.awsTags: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]}
               )

        # Tags from command line arguments
        if len(tableArgs.tagsTemp) > 0:
           for tag in tableArgs.tagsTemp: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[tableArgs.tagsTemp[tag]]}
               )

        subnets = ec2.describe_subnets( Filters=filters )

        jsonResult = ""
        if self.config.printResults or tableArgs.verbose or tableArgs.saveToFile: 
           jsonResult = Utils.dictToJson(subnets)

        resultTxt = ""
        if len(subnets["Subnets"]) >= 1:
            resultTxt = "The VPC " + Style.GREEN + vpcId + Style.RESET + " has " + Style.GREEN + str(len(subnets["Subnets"])) + Style.RESET + " Subnets\n"
            resultTxt = resultTxt + Utils.separator()  + "\n\n"

            idx_lin=0
            header      = ["#","Subnet Id", "Availability Zone", "CIDR Block", "Available IP Address Count"]
            if isPublic:
                header.append("Is Public?")
            if tableArgs.showTags:
                header.append("Tags")    
            prettyTable = PrettyTable(header)
            
            for subnet in subnets["Subnets"]:
                idx_lin += 1

                columns = [ str(idx_lin), subnet['SubnetId'], subnet['AvailabilityZone'],  subnet['CidrBlock'], subnet['AvailableIpAddressCount'] ]

                if isPublic:
                   resultIsPublic = self._checkSubnetIsPublic(subnet['SubnetId'], None)
                   result = "--- "
                   if resultIsPublic:
                      result = "PUBLIC"
                   columns.append(result)
                if tableArgs.showTags:
                   tags = Utils.formatPrintTags(subnet['Tags'])
                   columns.append(tags)

                prettyTable.addRow(columns)

            if (int(tableArgs.sortCol) - 1) > len(columns):
                tableArgs.sortCol = "1"
            prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
            prettyTable.ascendingOrder(not tableArgs.desc)
            result = "VPC...: " + Style.GREEN + vpcId + Style.RESET + "\n\n" + prettyTable.printMe("listSubnetsVpc",self.config.tableLineSeparator, tableArgs)
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_SUBNETS_VPC]["name"], result, self.config, jsonResult, True, tableArgs)
        else:
           resultTxt = ""
           try: 
             vpc = ec2.describe_vpcs(VpcIds=[vpcId])
             if not self.config.awsTagsToFilter():
                resultTxt = "VPC was found, but there's no Subnet associated. " + Emoticons.ops()
             else:   
                resultTxt = "VPC was found, but there's no Subnet associated or either did not pass the Filters Tag! " + Emoticons.ops()
           except ClientError as e:
             if "InvalidVpcID.NotFound" == e.response['Error']['Code']:
                resultTxt = "VPC with the Id " + Style.GREEN + vpcId + Style.RESET + " was not found! " + Emoticons.ops() +  Style.RESET
             else:    
                resultTxt = "Exception! ❌💣 " + Style.GREEN + e.response['Error']['Code'] + Style.RESET
           except:
             resultTxt = sys.exc_info + " ❌💣 "
           return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_SUBNETS_VPC]["name"], resultTxt, self.config, "", True, tableArgs)

    # List all VPCs
    def listVpc(self):
        ec2 = self.botoSession().client('ec2')

        tableArgs = TableArgs()
        tableArgs.setArguments(self.config.commandArguments)

        listSubnets    = False
        subnetIsPublic = False

        if "subnets" in self.config.commandArguments:
           listSubnets    = True
        if "ispublic" in self.config.commandArguments:
           subnetIsPublic = True   

        filters = []
        if self.config.awsTagsToFilter():
           # Tags from Environment
           for tag in self.config.awsTags: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]}
               )

        # Tags from command line arguments
        if len(tableArgs.tagsTemp) > 0:
           for tag in tableArgs.tagsTemp: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[tableArgs.tagsTemp[tag]]}
               )
        vpcs = ec2.describe_vpcs(Filters=filters)

        jsonResult = ""
        if self.config.printResults or tableArgs.verbose or tableArgs.saveToFile:
           jsonResult = Utils.dictToJson(vpcs)

        data         = {}
        data["vpcs"] = []
        resultTxt = ""
        if len(vpcs["Vpcs"]) >= 1:
            resultTxt = "List of VPCs, total of " + Style.GREEN + str(len(vpcs["Vpcs"])) + Style.RESET + "\n"
            resultTxt = resultTxt + Utils.separator()  + "\n\n"

            idx_lin=0
            header = ["#","VPC Id", "CIDR Block", "Default"]
            if listSubnets:
               header = ["#","VPC Id", "CIDR Block", "Default", "Subnet Id", "Subnet CIDR Block", "AZ", "Available IP Count"] 
               if subnetIsPublic:
                  header.append("Is Public?")
            if tableArgs.showTags:
               header.append("VPC Tags")       

            prettyTable = PrettyTable(header)
            for vpc in vpcs["Vpcs"]:
                idx_lin += 1

                vpcId       = vpc['VpcId']
                cidrBlock   = vpc['CidrBlock']
                default     = "Default" if vpc['IsDefault'] else "---"
                columns     = None
                subnetsDict = []
                azs         = []
                # Asked to list the subnets of each VPC?
                if listSubnets:
                   subnets = ec2.describe_subnets( Filters=[{'Name': 'vpc-id','Values': [vpc['VpcId']]}])
                   idx_subnet_lin = 0

                   for subnet in subnets["Subnets"]:
                       idx_subnet_lin += 1
                       lblSubnet = subnet['SubnetId'] + "  (" + str(idx_subnet_lin) + ")"
                       cidrBlockSubnet = subnet['CidrBlock']
                       az = subnet['AvailabilityZone']
                       availableIpdAddressCount = subnet['AvailableIpAddressCount']
                       
                       if az not in azs:
                          azs.append(az)

                       columns = [ idx_lin, vpcId, cidrBlock, default, lblSubnet, cidrBlockSubnet, az, availableIpdAddressCount ]

                       # Asked to inform if the Subnet is Public? 
                       resultSubnetIsPublic = None
                       if subnetIsPublic:
                          resultSubnetIsPublic = "--- "
                          if self._checkSubnetIsPublic(subnet['SubnetId'], None) :
                             resultSubnetIsPublic = "PUBLIC"
                          columns.append(resultSubnetIsPublic)
                       if tableArgs.showTags:
                          tags = Utils.formatPrintTags(vpc['Tags'])
                          columns.append(tags)   

                       prettyTable.addRow(columns)
                       
                       publicPrivateSubnet = ""
                       if resultSubnetIsPublic:
                          publicPrivateSubnet = resultSubnetIsPublic
                       
                       subnetsDict.append({
                          "subnetId": subnet['SubnetId'],
                          "cidrBlock": subnet['CidrBlock'],
                          "az": subnet['AvailabilityZone'],
                          "availableIpAddressCount": subnet['AvailableIpAddressCount'],
                          "public": publicPrivateSubnet
                       })

                   prettyTable.addSeparatorGroup()    
                else:   
                   columns = [ idx_lin, vpcId, cidrBlock, default ]
                   if tableArgs.showTags:
                      tags = Utils.formatPrintTags(vpc['Tags'])
                      columns.append(tags)
                   prettyTable.addRow(columns)

                vpcName = ""
                if 'Tags' in vpc:
                  for tag in vpc['Tags']:
                     if tag["Key"].lower() == "name":
                        vpcName = tag["Value"]
                        break
                
                data["vpcs"].append({
                   "vpcId": vpcId,
                   "cidrBlock": cidrBlock,
                   "name": vpcName,
                   "default": vpc['IsDefault'],
                   "azs": azs,
                   "subnets": subnetsDict
                })

            if (int(tableArgs.sortCol) - 1) > len(columns):
                sortCol = "1"

            prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
            prettyTable.ascendingOrder(not tableArgs.desc)
            result = "VPCs " + "\n\n" + prettyTable.printMe("listVpc",self.config.tableLineSeparator, tableArgs)
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_VPC]["name"], result, self.config, jsonResult, True, tableArgs)    
        else:
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_VPC]["name"], " No VPC found " + Emoticons.ops(), self.config, jsonResult, True, tableArgs)

    
    # Generate Data to create a traditional AWS VPC Networking diagram(VPCs, Subnets, CIDRBlocks, RTs, NACLs, etc.) for DrawIo CVS Import
    def drawNetworking(self):
        ec2 = self.botoSession().client('ec2')

        tableArgs = TableArgs()
        tableArgs.setArguments(self.config.commandArguments)

        addRouteTables = False
        addNacls       = False
        addTgws        = False
        addRtTgws      = False
        subnetIsPublic = True

        if "routetables" in self.config.commandArguments:
           addRouteTables = True
        if "nacls" in self.config.commandArguments:
           addNacls       = True
        if "tgws" in self.config.commandArguments:
           addTgws        = True
        if "rttgws" in self.config.commandArguments:
           addRtTgws      = True  
        if "all" in self.config.commandArguments:
           addRouteTables = True
           addNacls       = True
           addTgws        = True
           addRtTgws      = True

        filters = []
        if self.config.awsTagsToFilter():
           # Tags from Environment
           for tag in self.config.awsTags: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]}
               )

        # Tags from command line arguments
        if len(tableArgs.tagsTemp) > 0:
           for tag in tableArgs.tagsTemp: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[tableArgs.tagsTemp[tag]]}
               )
        vpcs = ec2.describe_vpcs(Filters=filters)

        data         = {}
        data["vpcs"] = []
        data["tgws"] = []
        resultTxt = ""

        if addTgws:
           # Attachments TransitGateway
           attachmentByTgw = {}
           #attachments     = ec2.describe_transit_gateway_attachments(Filters=filters)
           attachments     = ec2.describe_transit_gateway_attachments()
           for attachment in attachments["TransitGatewayAttachments"]:

              if not attachment["TransitGatewayId"] in attachmentByTgw:
                 attachmentByTgw[attachment["TransitGatewayId"]] = []
               
              attachmentByTgw[attachment["TransitGatewayId"]].append({
                  "transitGatewayAttachmentId": attachment["TransitGatewayAttachmentId"],
                  "transitGatewayOwnerId": attachment["TransitGatewayOwnerId"],
                  "resourceOwnerId": attachment["ResourceOwnerId"],
                  "resourceType": attachment["ResourceType"],
                  "resourceId": attachment["ResourceId"],
                  "state": attachment["State"],
                  "association": {
                     "transitGatewayRouteTableId": attachment["Association"]["TransitGatewayRouteTableId"],
                     "state": attachment["Association"]["State"]
                  }
              })
           # Route Tables (with its Routes) for TransitGateway
           tgwRouteTableByTgw = {}
           #tgwRouteTables = ec2.describe_transit_gateway_route_tables(Filters=filters)
           tgwRouteTables = ec2.describe_transit_gateway_route_tables()
           for tgwRt in tgwRouteTables["TransitGatewayRouteTables"]:
               if not tgwRt["TransitGatewayId"] in tgwRouteTableByTgw:
                  tgwRouteTableByTgw[tgwRt["TransitGatewayId"]] = []

               # Search Routes for the TGW Route Table
               routesTgwRouteTable = ec2.search_transit_gateway_routes(TransitGatewayRouteTableId=tgwRt["TransitGatewayRouteTableId"],
                     Filters=[
                        {
                              'Name': 'state',
                              'Values': [
                                 'active',
                              ]
                        },
                     ])
               
               routes = []
               for route in routesTgwRouteTable["Routes"]:
                  attchs = []
                  for att in route["TransitGatewayAttachments"]:
                      attchs.append({
                         "transitGatewayAttachmentId": att["TransitGatewayAttachmentId"],
                         "resourceId": att["ResourceId"],
                         "resourceType": att["ResourceType"]
                      })
                  routes.append({
                      "destinationCidrBlock": route["DestinationCidrBlock"],
                      "attachments": attchs
                  })
               tgwRouteTableByTgw[tgwRt["TransitGatewayId"]].append({
                  "routeTableId": tgwRt["TransitGatewayRouteTableId"],
                  "routes": routes
               })

           tgws = ec2.describe_transit_gateways(Filters=filters)
           for tgw in tgws["TransitGateways"]:

               data["tgws"].append({
                  "transitGatewayId": tgw["TransitGatewayId"],
                  "description": tgw["Description"] if "Description" in tgw else "",
                  "state": tgw["State"],
                  "ownerId": tgw["OwnerId"],
                  "options": {
                     "associationDefaultRouteTableId": tgw["Options"]["AssociationDefaultRouteTableId"] if "AssociationDefaultRouteTableId" in tgw["Options"] else "",
                     "defaultRouteTablePropagation": tgw["Options"]["DefaultRouteTablePropagation"] if "DefaultRouteTablePropagation" in tgw["Options"] else "",
                     "propagationDefaultRouteTableId": tgw["Options"]["PropagationDefaultRouteTableId"] if "PropagationDefaultRouteTableId" in tgw["Options"] else ""
                  },
                  "attachments": attachmentByTgw[tgw["TransitGatewayId"]] if tgw["TransitGatewayId"] in attachmentByTgw else [],
                  "routeTables": tgwRouteTableByTgw[tgw["TransitGatewayId"]] if tgw["TransitGatewayId"] in tgwRouteTableByTgw else []
               })
        
        if len(vpcs["Vpcs"]) >= 1:
            resultTxt = "List of VPCs, total of " + Style.GREEN + str(len(vpcs["Vpcs"])) + Style.RESET + "\n"
            resultTxt = resultTxt + Utils.separator()  + "\n\n"

            idx_lin=0
            for vpc in vpcs["Vpcs"]:
                idx_lin += 1

                vpcId       = vpc['VpcId']
                cidrBlock   = vpc['CidrBlock']
                default     = "Default" if vpc['IsDefault'] else "---"
                subnetsDict = []
                azs         = []
                # Asked to list the subnets of each VPC?
                
                subnets = ec2.describe_subnets( Filters=[{'Name': 'vpc-id','Values': [vpc['VpcId']]}])
                idx_subnet_lin = 0
 
                for subnet in subnets["Subnets"]:
                   idx_subnet_lin          += 1
                   cidrBlockSubnet          = subnet['CidrBlock']
                   az                       = subnet['AvailabilityZone']
                   availableIpdAddressCount = subnet['AvailableIpAddressCount']
                   name                     = ""

                   if "Tags" in subnet:
                      for t in subnet["Tags"]:
                         if t["Key"].upper() == "NAME":
                            name = t["Value"]
                            break
                   
                   if az not in azs:
                     azs.append(az)

                   # Get its Route Tables if asked
                   routeTables = []
                   if addRouteTables:
                      rts = ec2.describe_route_tables( Filters=[{'Name': 'association.subnet-id','Values': [subnet['SubnetId']]}])
                      if len(rts["RouteTables"]) == 0:
                         # In this case, when the Subnet is not directly associate with a user's created Route Table
                         # Get the Main Route Table of the VPC where it is located this Subnet (default association)
                         rts = ec2.describe_route_tables( Filters=[{'Name': 'association.main','Values':['true']},{'Name': 'vpc-id','Values':[vpc['VpcId']]}])
                      for routeTable in rts["RouteTables"]:
                          routeTables.append({
                             "routeTableId": routeTable["RouteTableId"],
                             "routeTableId5Digts": routeTable["RouteTableId"][-5:]
                          })
                   
                   # Get its NACLs if asked
                   nacls = []
                   if addNacls:
                      ncls = ec2.describe_network_acls( Filters=[{'Name': 'association.subnet-id','Values': [subnet['SubnetId']]}])
                      for nacl in ncls["NetworkAcls"]:
                          nacls.append({
                             "naclId": nacl["NetworkAclId"],
                             "nacl5Digts": nacl["NetworkAclId"][-5:]
                          })
                  
                   # Check if the Subnet is Public or Private 
                   # Asked to inform if the Subnet is Public? 
                   if subnetIsPublic:
                      resultSubnetIsPublic = "--- "
                      if self._checkSubnetIsPublic(subnet['SubnetId'], None) :
                         resultSubnetIsPublic = "PUBLIC"
                   publicPrivateSubnet = True if resultSubnetIsPublic.upper() == "PUBLIC" else False
                   # For tests
                   #publicPrivateSubnet = "PRIVATE" if int(random.randint(0,1)) == 1 else "PUBLIC"

                   subnetsDict.append({
                     "subnetId": subnet['SubnetId'],
                     "name": name,
                     "cidrBlock": subnet['CidrBlock'],
                     "az": subnet['AvailabilityZone'],
                     "availableIpAddressCount": subnet['AvailableIpAddressCount'],
                     "public": publicPrivateSubnet,
                     "routeTables": routeTables,
                     "nacls": nacls
                   })

                vpcName = ""
                if 'Tags' in vpc:
                  for tag in vpc['Tags']:
                     if tag["Key"].lower() == "name":
                        vpcName = tag["Value"]
                        break
                
                data["vpcs"].append({
                   "vpcId": vpcId,
                   "cidrBlock": cidrBlock,
                   "name": vpcName,
                   "default": vpc['IsDefault'],
                   "azs": azs,
                   "subnets": subnetsDict
                })

        #if True:
            #data = Utils.loadJsonFile("d:\\Downloads\\data.json.json")
            #if os.path.exists("D:\\Downloads\\data.json"):
            #   os.remove("D:\\Downloads\\data.json")
            #Utils.saveDictAsJson("D:\\Downloads\\data.json",data)

            csvDrawIo = CsvDrawIoNetworking(self.config)
            file = csvDrawIo.generateNetworkingGraph('networkingDiagram_template.csv', data, addRouteTables, addNacls, addTgws, addRtTgws)
            #file = csvDrawIo.generateNetworkingGraph('networkingDiagram_template.csv', data, True, True, True, True)
            
            print("")
            print("  --> " + Style.IGREEN + file + Style.GREEN + " generated!" + Style.RESET)
            print("  --> " + Style.IGREEN + "The content is also in your Clipboard, just go to DrawIo Menu" + Style.IBLUE + " --> " + Style.IGREEN + "Arrange" + Style.IBLUE + " --> " + Style.IGREEN + "Insert" + Style.IBLUE + " --> " + Style.IGREEN + "Advanced" + Style.IBLUE + " --> " + Style.IGREEN + "CSV..." + Style.RESET)
            print("  --> " + Style.IGREEN + "Paste the CSV content generated here" + Style.RESET)
            print("") 
            return ""
        else:
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_VPC]["name"], " No VPC found " + Emoticons.ops(), self.config, jsonResult, True, tableArgs)

    # List all Route Tables
    def listRouteTables(self):
        ec2 = self.botoSession().client('ec2')

        tableArgs = TableArgs()
        tableArgs.setArguments(self.config.commandArguments)

        showRoutes       = False

        if "show-routes" in self.config.commandArguments:
           showRoutes = True   

        data                = {}
        data["RouteTables"] = []
        filters             = []
        if self.config.awsTagsToFilter():
           # Tags from Environment
           for tag in self.config.awsTags: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]}
               )

        # Tags from command line arguments
        if len(tableArgs.tagsTemp) > 0:
           for tag in tableArgs.tagsTemp: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[tableArgs.tagsTemp[tag]]}
               )
        routeTables = ec2.describe_route_tables(Filters=filters)

        jsonResult = ""
        if self.config.printResults or tableArgs.verbose or tableArgs.saveToFile:
           jsonResult = Utils.dictToJson(routeTables)

        resultTxt = ""
        if len(routeTables["RouteTables"]) >= 1:
            resultTxt = "List of Route Tables, total of " + Style.GREEN + str(len(routeTables["RouteTables"])) + Style.RESET + "\n"
            resultTxt = resultTxt + Utils.separator()  + "\n\n"

            idx_lin=0
            header = ["#","VPC Id", "Route Table Id", "Associations"]
            if showRoutes:
               header = ["#","VPC Id", "Route Table Id", "Associations", "Routes"]   
            if tableArgs.showTags:
               header.append("Route Table Tags")       

            prettyTable = PrettyTable(header)
            for routeTable in routeTables["RouteTables"]:
                idx_lin += 1

                vpcId             = routeTable['VpcId']
                routeTableId      = routeTable['RouteTableId']
                columns           = [ idx_lin, vpcId, routeTableId ]
                associationsLines = []
                routeLines        = []

                
                subnets = []
                routes  = []
                # Save all the Associations in an array
                if "Associations" in routeTable:
                   associationsLines, subnets = self.listRouteTablesAddAssociations(routeTable)
                # Save all the Routes in an array (if requested)
                if showRoutes:
                   if "Routes" in routeTable:
                       routeLines, routes = self.listRouteTablesAddRoutes(routeTable, prettyTable)

                data["RouteTables"].append({
                   "vpcId": routeTable['VpcId'],
                   "routeTableId": routeTable['RouteTableId'],
                   "routeTableIdNickName": routeTable['RouteTableId'][-5:],
                   "subnets": subnets,
                   "routes": routes
                })
                
                # Check which one of the Arrays is larger (n.o of occurrences)
                #  choose the larger to guide the interation checking if the other (smaller) has a line with the same index, when it does print it, otherwise fill it with spaces
                if len(associationsLines) > len (routeLines):
                   # In case there are more Associations then Routes
                   for i, assoc in enumerate(associationsLines):
                       columns.append(assoc)
                       if showRoutes:
                           if i < len(routeLines):
                              columns.append(routeLines[i])
                           else:
                              columns.append("") 
                       if tableArgs.showTags:
                          tags = Utils.formatPrintTags(routeTable['Tags'])
                          columns.append(tags)
                       prettyTable.addRow(columns)
                       columns = [ idx_lin, vpcId, routeTableId ]
                else:
                   # In case there are more Routes then Associations
                   for i, route in enumerate(routeLines):
                       if i < len(associationsLines):
                          columns.append(associationsLines[i])
                       else:
                          columns.append("")
                       columns.append(route)
                       if tableArgs.showTags:
                          tags = Utils.formatPrintTags(routeTable['Tags'])
                          columns.append(tags)
                       prettyTable.addRow(columns)
                       columns = [ idx_lin, vpcId, routeTableId ]

                prettyTable.addSeparatorGroup() 

            if (int(tableArgs.sortCol) - 1) > len(columns):
                sortCol = "1"

            #print(Utils.saveDictAsJson("Json_Test_Rts.json",data))
            
            if tableArgs.csvDrawIo:
               csvDrawIo = CsvDrawIo(self.config)
               file = csvDrawIo.generateListRouteTables('listRouteTables_template.csv', data)
               print("")
               print("  --> " + Style.IGREEN + file + Style.GREEN + " generated!" + Style.RESET)
               print("")

            prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
            prettyTable.ascendingOrder(not tableArgs.desc)
            result = "Route Tables " + "\n\n" + prettyTable.printMe("listRouteTables",self.config.tableLineSeparator, tableArgs)
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_ROUTE_TABLES]["name"], result, self.config, jsonResult, True, tableArgs)    
        else:
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_ROUTE_TABLES]["name"], " No Route Table found " + Emoticons.ops(), self.config, jsonResult, True, tableArgs)
   
    def listRouteTablesAddAssociations(self, routeTable):
       idx_associac_lin  = 0
       associationsLines = []
       subnets = []
       for association in routeTable["Associations"]:
          if "SubnetId" in association:
             idx_associac_lin += 1
             subnetId = association['SubnetId'] + "  (" + str(idx_associac_lin) + ")"
             if "AssociationState" in association:
                  subnetId += subnet + " ( %s)".format(association["AssociationState"]["State"])
             associationsLines.append(subnetId)
             subnets.append({
                "subnetId": association['SubnetId'],
                "main": association["Main"],
                "routeTableAssociationId": association["RouteTableAssociationId"]
             })
          else:
             subnetId = ""
             associationsLines.append(subnetId)
       return associationsLines, subnets

    def listRouteTablesAddRoutes(self, routeTable, prettyTable):
       idx_route_lin  = 0
       routeLines     = []
       chgdHeader     = False
       routes         = []
       for route in routeTable["Routes"]:
          idx_route_lin += 1

          cidrblock = ""
          if "DestinationCidrBlock" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["DestinationCidrBlock"]
             cidrblock            = route["DestinationCidrBlock"]
          elif "DestinationIpv6CidrBlock" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["DestinationIpv6CidrBlock"]
             cidrblock            = route["DestinationIpv6CidrBlock"]
          elif "DestinationPrefixListId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["DestinationPrefixListId"]
             cidrblock            = route["DestinationPrefixListId"]
          elif "EgressOnlyInternetGatewayId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["EgressOnlyInternetGatewayId"]
             cidrblock            = route["EgressOnlyInternetGatewayId"]
          elif "GatewayId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["GatewayId"]
             cidrblock            = route["GatewayId"]
          elif "InstanceId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["InstanceId"]
             cidrblock            = route["InstanceId"]
          elif "InstanceOwnerId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["InstanceOwnerId"]
             cidrblock            = route["InstanceOwnerId"]
          elif "NatGatewayId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["NatGatewayId"]
             cidrblock            = route["NatGatewayId"]
          elif "TransitGatewayId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["TransitGatewayId"]
             cidrblock            = route["TransitGatewayId"]
          elif "LocalGatewayId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["LocalGatewayId"]
             cidrblock            = route["LocalGatewayId"]
          elif "CarrierGatewayId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["CarrierGatewayId"]
             cidrblock            = route["CarrierGatewayId"]
          elif "NetworkInterfaceId" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["NetworkInterfaceId"]
             cidrblock            = route["NetworkInterfaceId"]
          elif "Origin" in route:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + route["Origin"]
             cidrblock            = route["Origin"]
          else:
             destinationCidrBlock = "(" + "{:02d}".format(idx_route_lin) + ") " + str(route)
             cidrblock            = str(route)
         
          target     = ""
          state      = route["State"]
          targetType = ""  
          if "TransitGatewayId" in route:
             target     = route['TransitGatewayId'] 
             targetType = "TransitGateway"
          elif "GatewayId" in route:
             target     = route['GatewayId']
             targetType = "Gateway"
          elif "InstanceId" in route:
             target     = route['InstanceId']
             targetType = "Instance"
          elif "NatGatewayId" in route:
             target     = route['NatGatewayId']
             targetType = "NatGateway"
          elif "VpcPeeringConnectionId" in route:
             target     = route['VpcPeeringConnectionId']
             targetType = "VPC Peering"
          else:
             target = str(route)

          line = destinationCidrBlock.ljust(24," ")  + " | " + target.ljust(25," ") + " | " + state.ljust(10," ")

          if not chgdHeader:
             entriesHeader = "CIDR Block".ljust(24," ")  + " | " + "Target".ljust(25," ") + " | " + "State".ljust(10," ")
             prettyTable.header = ["#","VPC Id", "Route Table Id", "Associations", entriesHeader]   
             chgdHeader = True

          routes.append({
             "target": target,
             "targetType": targetType,
             "cidrblock": cidrblock,
             "state": state
          })
          routeLines.append(line)
       return routeLines, routes

    # List Network Access Control List
    def listNacls(self):
        ec2 = self.botoSession().client('ec2')

        tableArgs = TableArgs()
        tableArgs.setArguments(self.config.commandArguments)

        showEntries  = False

        if "show-entries" in self.config.commandArguments:
           showEntries = True   

        filters = []
        if self.config.awsTagsToFilter():
           # Tags from Environment
           for tag in self.config.awsTags: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]}
               )

        # Tags from command line arguments
        if len(tableArgs.tagsTemp) > 0:
           for tag in tableArgs.tagsTemp: 
               filters.append(
                   {'Name':'tag:' + tag,'Values':[tableArgs.tagsTemp[tag]]}
               )
        nacls = ec2.describe_network_acls(Filters=filters)

        jsonResult = ""
        if self.config.printResults or tableArgs.verbose or tableArgs.saveToFile:
           jsonResult = Utils.dictToJson(nacls)

        resultTxt = ""
        if len(nacls["NetworkAcls"]) >= 1:
            resultTxt = "List of Route Tables, total of " + Style.GREEN + str(len(nacls["NetworkAcls"])) + Style.RESET + "\n"
            resultTxt = resultTxt + Utils.separator()  + "\n\n"

            idx_lin=0
            header = ["#","VPC Id", "NACL Id", "Associations"]
            if showEntries:
               header = ["#","VPC Id", "NACL Id", "Associations", "Entries"]   
            if tableArgs.showTags:
               header.append("NACL Table Tags")       

            prettyTable = PrettyTable(header)
            for nacl in nacls["NetworkAcls"]:
                idx_lin += 1

                vpcId             = nacl['VpcId']
                naclId            = nacl['NetworkAclId']
                columns           = [ idx_lin, vpcId, naclId ]
                associationsLines = []
                entryLines        = []

                # Save all the Associations in an array
                if "Associations" in nacl:
                   associationsLines = self.listNaclsAddAssociations(nacl)
                # Save all the Entries in an array (if requested)
                if showEntries:
                   if "Entries" in nacl:
                       entryLines = self.listNaclsAddEntries(nacl, prettyTable)
                
                # Check which one of the Arrays is larger (n.o of occurrences)
                #  choose the larger to guide the interation checking if the other (smaller) has a line with the same index, when it does print it, otherwise fill it with spaces
                if len(associationsLines) > len (entryLines):
                   # In case there are more Associations then Entries
                   for i, assoc in enumerate(associationsLines):
                       columns.append(assoc)
                       if showEntries:
                           if i < len(entryLines):
                              columns.append(entryLines[i])
                           else:
                              columns.append("") 
                       if tableArgs.showTags:
                          tags = Utils.formatPrintTags(nacl['Tags'])
                          columns.append(tags)
                       prettyTable.addRow(columns)
                       columns = [ idx_lin, vpcId, naclId ]
                else:
                   # In case there are more Entries then Associations
                   for i, entry in enumerate(entryLines):
                       if i < len(associationsLines):
                          columns.append(associationsLines[i])
                       else:
                          columns.append("")
                       columns.append(entry)
                       if tableArgs.showTags:
                          tags = Utils.formatPrintTags(nacl['Tags'])
                          columns.append(tags)
                       prettyTable.addRow(columns)
                       columns = [ idx_lin, vpcId, naclId ]

                prettyTable.addSeparatorGroup() 

            if (int(tableArgs.sortCol) - 1) > len(columns):
                sortCol = "1"

            prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
            prettyTable.ascendingOrder(not tableArgs.desc)

            if tableArgs.csvDrawIo:
               csvDrawIo = CsvDrawIo(self.config)
               file = csvDrawIo.generateListNacls('listNacls_template.csv', prettyTable.header, prettyTable.content)
               print("")
               print("  --> " + Style.IGREEN + file + Style.GREEN + " generated!" + Style.RESET)
               print("")

            result = "Network Access Control List " + "\n\n" + prettyTable.printMe("listNacls",self.config.tableLineSeparator, tableArgs)
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_ROUTE_TABLES]["name"], result, self.config, jsonResult, True, tableArgs)
        else:
            return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_ROUTE_TABLES]["name"], " No Network Access Control List found " + Emoticons.ops(), self.config, jsonResult, True, tableArgs)

    
    def listNaclsAddAssociations(self, nacl):
       idx_associac_lin  = 0
       associationsLines = []
       for association in nacl["Associations"]:
          if "SubnetId" in association:
             idx_associac_lin += 1
             subnetId = association['SubnetId'] + "  (" + str(idx_associac_lin) + ")"
             if "AssociationState" in association:
                  subnetId += subnet + " ( %s)".format(association["AssociationState"]["State"])
             associationsLines.append(subnetId)
          else:
             subnetId = ""
             associationsLines.append(subnetId)
       return associationsLines

    def listNaclsAddEntries(self, nacl, prettyTable):
       idx_route_lin  = 0
       entries        = []
       chgdHeader     = False
       for entry in nacl["Entries"]:
          idx_route_lin += 1
          cidrBlock  = "(" + "{:02d}".format(idx_route_lin) + ") " + entry["CidrBlock"]
          type       = "Egress" if entry["Egress"] else ""
          protocol   = str(entry["Protocol"])
          ruleAction = entry["RuleAction"] 
          ruleNumber = str(entry["RuleNumber"])
          line = cidrBlock.ljust(24," ")  + " | " + type.ljust(10," ") + " | " + ruleAction.ljust(8," ") + " | " + ruleNumber.ljust(5," ") + " | " + protocol.ljust(8," ")

          if not chgdHeader:
             entriesHeader = "CDIR Block".ljust(24," ")  + " | " + "Type".ljust(10," ") + " | " + "Action".ljust(8," ") + \
                             " | " + "Num".ljust(5," ") + " | " + "Protocol".ljust(8," ")
             prettyTable.header = ["#","VPC Id", "NACL Id", "Associations", entriesHeader]
             chgdHeader = True

          entries.append(line)
       return entries

    # List all Ec2 Instances
    def listEc2(self):
       ec2              = self.botoSession().client('ec2')
       instanceId       = None
       privateIPFilter  = None
       vpcIdFilter      = None
       subnetIdFilter   = None
       showSg           = False
       showVpc          = False
       showSubnet       = False
       showImage        = False
       showCpu          = False
       hideName         = False

       tableArgs = TableArgs()
       tableArgs.setArguments(self.config.commandArguments)
       if "," in self.config.commandArguments:
          for arg in self.config.commandArguments.split(","):
              if "i-" in arg:
                 instanceId = arg
              elif arg == "sg":
                 showSg = True
              elif arg == "vpc":
                 showVpc = True    
              elif arg == "subnet":
                 showSubnet = True   
              elif arg == "image":
                 showImage = True 
              elif arg == "cpu":
                 showCpu = True
              elif arg == "noname":
                 hideName = True
              elif arg.startswith("vpc-"):
                 vpcIdFilter = arg
              elif arg.startswith("subnet-"):
                 subnetIdFilter = arg              
              else:
                 # Check IP
                 if Utils.isNumber(arg[0:2]):
                    privateIPFilter = arg

       else:
          arg = self.config.commandArguments
          if "i-" in arg:
            instanceId = arg
          elif arg == "sg":
             showSg = True
          elif arg == "vpc":
             showVpc = True    
          elif arg == "subnet":
             showSubnet = True   
          elif arg == "image":
             showImage = True 
          elif arg == "cpu":
             showCpu = True
          elif arg == "noname": 
             hideName = True
          elif arg.startswith("vpc-"):
             vpcIdFilter = arg
             print(arg)
          elif arg.startswith("subnet-"):
             subnetIdFilter = arg   
          else:
             # Check IP
             if Utils.isNumber(arg[0:2]):
                privateIPFilter = arg


       filters=[]   

       if instanceId:
          filters.append({'Name': 'instance-id','Values': [instanceId]})   
       if privateIPFilter:   
          filters.append({'Name': 'private-ip-address','Values': [privateIPFilter]})
       if vpcIdFilter:   
          filters.append({'Name': 'vpc-id','Values': [vpcIdFilter]})
       if subnetIdFilter:   
          filters.append({'Name': 'subnet-id','Values': [subnetIdFilter]})         

       if self.config.awsTagsToFilter():
           # Environment Tags set
           for tag in self.config.awsTags: 
               filters.append({'Name':'tag:' + tag,'Values':[self.config.awsTags[tag]]})   

       # Tags from command line arguments
       if len(tableArgs.tagsTemp) > 0:
          for tag in tableArgs.tagsTemp: 
              filters.append({'Name':'tag:' + tag,'Values':[tableArgs.tagsTemp[tag]]})        

       reservations = ec2.describe_instances( Filters=filters )        

       jsonResult = ""
       if self.config.printResults or tableArgs.verbose or tableArgs.saveToFile: 
          jsonResult = Utils.dictToJson(reservations)

       if reservations and len(reservations["Reservations"]) > 0:
  
          idx_lin = 0

          if hideName:
             header = ["#","Instance Id",       "Type","Launch Time", "Private IP", "Public IP", "State"]
          else:   
             header = ["#","Instance Id","Name","Type","Launch Time", "Private IP", "Public IP", "State"]

          if showSg:             
             header.append("Security Groups")
          if showImage:          
             header.append("Image Id")
          if showCpu:            
             header.append("CPU Options")
          if showVpc:            
             header.append("VPC")
          if showSubnet:         
             header.append("Subnet")
          if tableArgs.showTags: 
             header.append("Tags")

          prettyTable = PrettyTable(header)

          totalListed = 0

          for instanceGroup in reservations["Reservations"]:
              for instance in instanceGroup["Instances"]:
                  idx_lin     += 1
                  totalListed += 1

                  if not hideName:
                     name = ""
                     for t in instance['Tags']:
                         if t["Key"] == "Name":
                            name = t["Value"]

                  launchTimeDate = ""
                  if instance['LaunchTime']:
                     launchTimeDate = instance['LaunchTime'].strftime("%d-%b-%y %H:%M")

                  privateIp = instance['PrivateIpAddress'] if "PrivateIpAddress" in instance else ""
                  publicIp  = instance["PublicIpAddress"] if "PublicIpAddress" in instance else ""
                  state     = instance["State"]["Name"]
                  state     = Style.CYAN + state + Style.RESET if state == "stopped" else state

                  if hideName:
                     columns = [ str(idx_lin), str(instance['InstanceId']), instance['InstanceType'], launchTimeDate, privateIp, publicIp, state ]
                  else:   
                     columns = [ str(idx_lin), str(instance['InstanceId']), name, instance['InstanceType'], launchTimeDate, privateIp, publicIp, state ]

                  if showSg:
                     securityGroups = ""
                     for sg in instance["SecurityGroups"]:             
                         if securityGroups != "": securityGroups += ","
                         securityGroups += sg["GroupId"]
                     columns.append(securityGroups)    
                  if showImage:          
                     columns.append(instance["ImageId"])       
                  if showCpu:   
                     cpu  = Style.MAGENTA + "Core:"    + Style.GREEN + str(instance["CpuOptions"]["CoreCount"]) + Style.RESET + " "
                     cpu += Style.MAGENTA + "Threads:" + Style.GREEN + str(instance["CpuOptions"]["ThreadsPerCore"])
                     columns.append(cpu)       
                  if showVpc:            
                     vpc = instance["VpcId"]
                     columns.append(vpc)
                  if showSubnet:         
                     subnet = instance["SubnetId"]
                     columns.append(subnet)
                  if tableArgs.showTags:
                     tags = Utils.formatPrintTags(instance['Tags'])
                     columns.append(tags)

                  prettyTable.addRow(columns)

          resultTxt = ""

          if instanceId:
             if resultTxt != "": resultTxt = resultTxt + "\n"
             resultTxt = resultTxt + "      Instance Id........: " + Style.GREEN + instanceId + Style.RESET
          if privateIPFilter:   
             if resultTxt != "": resultTxt = resultTxt + "\n"
             resultTxt = resultTxt + "      Private IP Address.: " + Style.GREEN + privateIPFilter + Style.RESET
          if vpcIdFilter:   
             if resultTxt != "": resultTxt = resultTxt + "\n"
             resultTxt = resultTxt + "      VPC Id.............: " + Style.GREEN + vpcIdFilter + Style.RESET
          if subnetIdFilter:   
             if resultTxt != "": resultTxt = resultTxt + "\n"
             resultTxt = resultTxt + "      Subnet Id..........: " + Style.GREEN + subnetIdFilter + Style.RESET

          resultTxt = resultTxt + "\n\n Total of EC2s...: " + Style.GREEN + format(totalListed,",") + Style.RESET   

          if (int(tableArgs.sortCol) - 1) > len(columns):
              tableArgs.sortCol = "1"
          prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
          prettyTable.ascendingOrder(not tableArgs.desc)

          result = resultTxt + "\n\n" + prettyTable.printMe("listEc2",self.config.tableLineSeparator, tableArgs)
          return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_EC2]["name"], result, self.config,jsonResult, True, tableArgs), prettyTable.content
       else:
          resultTxt = ""
          resultTxt = "No EC2 Instance was found. " + Emoticons.ops()
          return Utils.formatResult(Functions.FUNCTIONS[Functions.LIST_EC2]["name"], resultTxt, self.config, "", True, tableArgs), None

    def botoSession(self):
        return self.config.botoSession()

if __name__ == '__main__':
    config = Config()
    config.awsProfile = "ecomm"
    config.awsRegion = "eu-central-1"
    config.printResults = True
    #config.awsTags["Environment"] = "production"
    config.tableLineSeparator = False
    
    
    # subnets, is-public (only with subnets active)
    # sort1,desc (when requested together with subnets, the group lines will be omitted)
    if len(sys.argv) > 1:
       config.commandArguments = sys.argv[1]
    pyEc2CP = PyEc2CP(config)
    report = pyEc2CP.listEc2()
    Utils.addToClipboard(report)
    print(report)

    # TODO:
    # findEc2 (by IP, by InstanceId, by Tag, etc.)
    # Result show: IP, t3.medium, region, subnet, vpc, security group, (attached Load Balancer optional), (Security Groups Rules optional)



from diagrams import Cluster, Edge, Node, Diagram
from diagrams.aws.compute import EC2
from diagrams.aws.database import RDS
from diagrams.aws.network import ELB
from diagrams.aws.network import VPC
from diagrams.aws.network import PublicSubnet
from diagrams.aws.network import PrivateSubnet
from pyawscp.Utils import Utils, Style
from pyawscp.PyEc2Cp import PyEc2CP
from pyawscp.PyS3Cp import PyS3CP
from pyawscp.Config import Config

class DiagramMgnt:

    def __init__(self, config):
        self.config = config
        """ with Diagram("listEc2", show=False):
             for idx_row, row in enumerate(self.content):
                 for idx_col, val in enumerate(row):
                     ELB(val) """
    
    def diagramVpcEc2(self):
        self.config.commandArguments = "sort9,vpc,subnet"
        pyEc2Cp = PyEc2CP(self.config)
        report, content = pyEc2Cp.listEc2()
        if content:
            EC2_INST_COL       = 1
            EC2_NAME_COL       = 2
            EC2_TYPE_COL       = 3
            EC2_LAUNCHTIME_COL = 4
            EC2_PRIVATEIP_COL  = 5
            EC2_PUBLICIP_COL   = 6
            EC2_STATE_COL      = 7
            VPC_COL            = 8
            SUBNET_COL         = 9

            data = {}
            for idx_row, row in enumerate(content):
                for idx_col, col in enumerate(row):
                    if idx_col == VPC_COL:
                       vpc = row[VPC_COL]
                data[vpc] = {}
            
            for idx_row, row in enumerate(content):
                for idx_col, col in enumerate(row):
                    if idx_col == VPC_COL:
                       vpc = row[VPC_COL]
                    if idx_col == SUBNET_COL:
                       subnet = row[SUBNET_COL]   
                data[vpc][subnet] = {}
            
            for idx_row, row in enumerate(content):
                for idx_col, col in enumerate(row):
                    if idx_col == VPC_COL:
                       vpc = row[VPC_COL]
                    if idx_col == SUBNET_COL:
                       subnet = row[SUBNET_COL]   
                    if idx_col == EC2_INST_COL:
                       instanceId = row[EC2_INST_COL]
                    if idx_col == EC2_NAME_COL:
                       name = row[EC2_NAME_COL]
                    if idx_col == EC2_TYPE_COL:
                       type = row[EC2_TYPE_COL]
                    if idx_col == EC2_PRIVATEIP_COL:
                       privateIp = row[EC2_PRIVATEIP_COL]
                    if idx_col == EC2_PUBLICIP_COL:
                       publicIp = row[EC2_PUBLICIP_COL]
                    if idx_col == EC2_STATE_COL:
                       state = row[EC2_STATE_COL]

                data[vpc][subnet][instanceId] = {
                    "Name": name,"Type": type, "PrivateIp": privateIp, "PublicIp": publicIp, "State": state
                }

            
            with Diagram("ListEc2", show=False, direction="TB") as builtDiagram:
                for vpc in data:
                    with Cluster(vpc):
                        for subnet in data[vpc]:
                            with Cluster(subnet):

                                ec2s = []
                                for ec2 in data[vpc][subnet]:
                                    ec2s.append(ec2)

                                cols       = 4
                                numberEc2s = len(ec2s)
                                rows       =  numberEc2s // cols
                                if (numberEc2s / cols) > (numberEc2s // cols):
                                   rows += 1
                                
                                index = 0
                                for row in range(rows):
                                    for col in range(cols):
                                        index += 1
                                        if index < len(ec2s):
                                            
                                           EC2(ec2s[index-1])
                                           
                                    
                                
                                

            
            #  with Diagram("ListEc2", show=False, direction="TB") as builtDiagram:
            #     for vpc in data:
            #         with Cluster(vpc):

            #             for subnet in data[vpc]:
            #                 with Cluster(subnet):

            #                     ec2s = []
            #                     for ec2 in data[vpc][subnet]:
            #                         ec2s.append(ec2)

            #                     print(vpc)
            #                     print(subnet)
            #                     print(ec2s)   
            #                     print("-------------------------")

            #                     shapes         = ec2s
            #                     num_shapes     = len(shapes)
            #                     shapes_per_row = 3
            #                     num_of_rows    = int(num_shapes / shapes_per_row) + (num_shapes % shapes_per_row > 0)

            #                     for row in range(num_of_rows)[::-1]:
            #                         items_in_row = shapes_per_row - (row+1) * shapes_per_row // num_shapes
            #                         shapes_i = row * shapes_per_row
            #                         node_list = ['EC2(\'' + shapes[shapes_i+item_num] + '\') - Edge(penwidth="0.0")'
            #                              for item_num in range(items_in_row)[:-1]
            #                         ] + ['EC2(\'' + shapes[shapes_i+items_in_row-1] + '\')']

            #                         print(node_list)

            #                         node_row = "-".join(node_list)
            #                         eval(node_row)
                                

# ['EC2(\'i-00b634c2643283086\') - Edge(penwidth="0.0")', "EC2('i-0996ac119c6ee0895')"]
# ['EC2(\'i-0becc277226e2053d\') - Edge(penwidth="0.0")', 'EC2(\'i-07ba3f05755dc725d\') - Edge(penwidth="0.0")', "EC2('i-0c7724d072ba3a1b4')"]
# ['EC2(\'i-0194df0d6a568fb7d\') - Edge(penwidth="0.0")', 'EC2(\'i-0fe280bce8efcda6d\') - Edge(penwidth="0.0")', "EC2('i-044aeb4b11e8901c5')"]
# ['EC2(\'i-04afcb7171c79b2d5\') - Edge(penwidth="0.0")', 'EC2(\'i-0ec5dce0be7aabdc7\') - Edge(penwidth="0.0")', "EC2('i-0ae56529c226e4c62')"]

                                   
            # builtDiagram

            # with Diagram("ListEc2", show=False, direction="TB"):
            #     for vpc in data:
            #         with Cluster(vpc):
            #             for subnet in data[vpc]:
            #                 with Cluster(subnet):
            #                     for ec2 in data[vpc][subnet]:
            #                         EC2(ec2) 
        else:
            print("No Content to Diagram!")
    
        





# coding=utf-8

import sys, cmd, os, ast, signal
from datetime import datetime
from os.path import expanduser
from arnparse import arnparse
import configparser
import json
import xlsxwriter
import dominate
from dominate.tags import *

from pyawscp.Config import Config
from pyawscp.Utils import Utils, Style, Emoticons
from pyawscp.Functions import Functions

TRANSFER_S3  = "TRANSFER_S3"
PREFERENCES = "PREFERENCES" 

# # python -m pyawscp.JustOneShot
class DoWhateverYouSupposeYouShouldDo:

    def __init__(self):
        self.preferences = Config()
        if self.preferences.isValidCookieTempCredentials():
           roleActived = True
           intro  = "\n"
           intro += Style.GREEN + "  " + Emoticons.cookie() + Emoticons.cookie() + Emoticons.cookie() + " " + Emoticons.pointRight() + \
                    " AWS Temporary "+Style.IBLUE+"Session"+Style.GREEN+" Credentials is still "+Style.IBLUE+"valid"+Style.GREEN+" and activated!"
           intro += "\n"
           intro += Style.GREEN + "            Expires at: " + Style.IBLUE + self.preferences.cookieTempCredentials().humanReadFormatExpirationDate() + " (Remaining: " + str(self.preferences.cookieTempCredentials().timeToExpire()) + ")"
           intro += "\n"
           print(intro)
    
        configFileIni = configparser.ConfigParser()
        configFileIni.read(expanduser("~") + "/.pyawscp/pyawscp.ini")
        self.preferences.awsProfile               = configFileIni[PREFERENCES]["aws-profile"]
        self.preferences.awsRegion                = configFileIni[PREFERENCES]["aws-region"]
        self.preferences.awsTags                  = ast.literal_eval(configFileIni[PREFERENCES]["aws-tags"])
        self.preferences.printResults             = configFileIni[PREFERENCES]["print-results"] in ['True','true']
        self.preferences.tableLineSeparator       = configFileIni[PREFERENCES]["table-line-separator"] in ['True','true']
        self.preferences.uploadChunkSizeMultipart = configFileIni[TRANSFER_S3]["chunkSize"]
        self.preferences.uploadThresholdMultipart = configFileIni[TRANSFER_S3]["threshold"]
    
        if "assume-role" in configFileIni[PREFERENCES]:
           self.preferences.assumeRole           = configFileIni[PREFERENCES]["assume-role"]
        else:
           self.preferences.assumeRole           = ""
        if "mfa-serial" in configFileIni[PREFERENCES]:
           self.preferences.mfaSerial           = configFileIni[PREFERENCES]["mfa-serial"]
        else:
           self.preferences.mfaSerial           = ""

    def botoSession(self):
        return self.preferences.botoSession()
    


    def doIt(self):
        self.getAllResources()

        

    ## List Resources 
    ## BEGIN
    def getAllResources(self):
        client = self.botoSession().client('resourcegroupstaggingapi')

        resourceDict              = {}
        resourceDict["resources"] = []

        pages     = 1
        resources = client.get_resources()
        self.processAllResourcesPage(resources, pages, resourceDict)
        tokenPagination = resources["PaginationToken"] if "PaginationToken" in resources else None
        
        if tokenPagination:
           while True:
              pages    += 1
              resources = client.get_resources(PaginationToken=tokenPagination)
              self.processAllResourcesPage(resources, pages, resourceDict)
              tokenPagination = resources["PaginationToken"] if "PaginationToken" in resources else None
              if not tokenPagination:
                 break
              # if pages > 1:
              #    break
        
        print(Style.IGREEN + "Total Pages Processed.....: " + str(pages))
        print(Style.GREEN)

        # Grouping
        totalByResourceType = []
        totalByService      = []
        totalBy             = {}

        for resource in resourceDict["resources"]:
            if not resource["resourceType"] in totalByResourceType:
               totalByResourceType.append(resource["resourceType"]) 

            if not resource["service"] in totalByService:
               totalByService.append(resource["service"])

            awsService = "{}_{}".format(resource["service"],resource["resourceType"])
            if  awsService in totalBy:
                totalBy[awsService]["total"] = totalBy[awsService]["total"] + 1
            else:
                totalBy[awsService] = {
                    "total": 1,
                    "resourceType": resource["resourceType"],
                    "service": resource["service"]
                }

        
        jsonContent = Utils.dictToJson(resourceDict)
        #print(jsonContent)
        print("Writing JSON List AWS Resources...")
        with open("d:\Downloads\\resources_{}.json".format(Utils.labelTimestamp()), 'w') as fout:
            fout.write(jsonContent)
            fout.flush()

        # Write an Excel
        print("Writing Excel List AWS Resources...")
        workbook = xlsxwriter.Workbook("d:\Downloads\\resources_{}.xlsx".format(Utils.labelTimestamp()))
        worksheet = workbook.add_worksheet("AWS Resources List")
        worksheet.write("A1", "Service")
        worksheet.write("B1", "Resource Type")
        worksheet.write("C1", "Name")
        worksheet.write("D1", "Account")
        worksheet.write("E1", "Region")
        worksheet.write("F1", "ARN")
        excelRow = 1
        for resource in resourceDict["resources"]:
            excelRow += 1
            worksheet.write("A" + str(excelRow), resource["service"])
            worksheet.write("B" + str(excelRow), resource["resourceType"])
            worksheet.write("C" + str(excelRow), resource["name"])
            worksheet.write("D" + str(excelRow), resource["accountId"])
            worksheet.write("E" + str(excelRow), resource["region"])
            worksheet.write("F" + str(excelRow), resource["arn"])

        excelRow = 1    
        worksheetSummary = workbook.add_worksheet("AWS Resources Summary")
        worksheetSummary.write("A1", "Service")
        worksheetSummary.write("B1", "Resource Type")
        worksheetSummary.write("C1", "Total")
        for key in totalBy:
            excelRow += 1
            worksheetSummary.write("A" + str(excelRow), totalBy[key]["service"])
            worksheetSummary.write("B" + str(excelRow), totalBy[key]["resourceType"])
            worksheetSummary.write("C" + str(excelRow), totalBy[key]["total"])
        workbook.close()
        
        # Write an HTML Summary
        h = html()
        with h.add(body()):
            with div(id='content'):
                attr(style='font-family:Arial')
                hr()
                h1('AWS Resources Summary')
                with table():
                    attr(cellpadding=3)
                    attr(cellspacing=0)
                    attr(style='width:100%;border:1px black solid;font-family:Arial;font-size: 12px;')
                    lheader = tr()
                    with lheader:
                        attr(align="center")
                        attr(style="background:black;color:white;font-weight:bolder;font-size:15px")
                        with td('Service'):
                            pass
                        with td('Resource Type'):
                            pass
                        with td('Total'):
                            pass
                    colorEven = "#D3D3D3"
                    colorOdd  = "#FFFFFF"
                    idx       = 0
                    for key in totalBy:
                        lineBackColor  = colorEven if idx % 2 == 0 else colorOdd
                        l = tr()
                        with l:
                            styleTd = "font-size:12px;font-family:Consolas;background:{background};color:black;font-weight:bolder;border-top:1px solid black;border-right:1px solid black".format(background=lineBackColor)
                            with td(totalBy[key]["service"]):
                                attr(style=styleTd)
                            with td(totalBy[key]["resourceType"] if totalBy[key]["resourceType"] else ""):
                                attr(style=styleTd)
                            with td(totalBy[key]["total"]):
                                attr(style=styleTd)
                        idx += 1

        print("Writing HTML Summary AWS Resources...")
        with open("d:\Downloads\\resources_summary_{}.html".format(Utils.labelTimestamp()), 'w') as fout:
        #with open("d:\Downloads\\resources_summary.html", 'w') as fout:
             fout.write(str(h))
        
        # Write Complete HTML List
        h = html()
        with h.add(body()):
            with div(id='content'):
                attr(style='font-family:Arial')
                hr()
                h1('AWS Resources')
                with table():
                    attr(cellpadding=3)
                    attr(cellspacing=0)
                    attr(style='width:100%;border:1px black solid;font-family:Arial;font-size: 12px;')
                    lheader = tr()
                    with lheader:
                        attr(align="center")
                        attr(style="background:black;color:white;font-weight:bolder;font-size:15px")
                        with td('#'):
                            pass
                        with td('Service'):
                            pass
                        with td('Resource Type'):
                            pass
                        with td('Name'):
                            pass
                        with td('Account'):
                            pass
                        with td('Region'):
                            pass
                        with td('ARN'):
                            pass
                    
                    colorEven = "#D3D3D3"
                    colorOdd  = "#FFFFFF"
                    idx       = 0
                    for resource in resourceDict["resources"]:
                        lineBackColor  = colorEven if idx % 2 == 0 else colorOdd
                        l = tr()
                        with l:
                            styleTd = "font-size:12px;font-family:Consolas;background:{background};color:black;font-weight:bolder;border-top:1px solid black;border-right:1px solid black".format(background=lineBackColor)
                            with td(str(idx+1)):
                                attr(style=styleTd)
                            with td(str(resource["service"])):
                                attr(style=styleTd)
                            with td(str(resource["resourceType"])):
                                attr(style=styleTd)
                            with td(str(resource["name"])):
                                attr(style=styleTd)
                            with td(str(resource["accountId"])):
                                attr(style=styleTd)
                            with td(str(resource["region"])):
                                attr(style=styleTd)
                            with td(str(resource["arn"])):
                                attr(style=styleTd)
                        idx += 1

        print("Writing HTML List AWS Resources...")
        with open("d:\Downloads\\resources_{}.html".format(Utils.labelTimestamp()), 'w') as fout:
        #with open("d:\Downloads\\resources.html", 'w') as fout:
             fout.write(str(h))

    def processAllResourcesPage(self, resources, pages, resourceDict):
        print(Style.IGREEN + 'Processing page... '+ str(pages) + Style.RESET)

        for resource in resources["ResourceTagMappingList"]:

            arn           = arnparse(resource["ResourceARN"])
            name          = ""
            service       = arn.service
            region        = arn.region
            accountId     = arn.account_id
            resource_type = arn.resource_type 

            #print("\033[32mService:\033[96m{}\n\033[32mName:\033[96m{}\n\033[32mResource Type:\033[96m{}\n".format(service, name, resource_type))

            for tag in resource["Tags"]:
                if "NAME" == tag["Key"].upper():
                    name = tag["Value"]
                    break
            
            resourceDict["resources"].append({
                "arn": resource["ResourceARN"],
                "name": name,
                "service": service,
                "region": region,
                "accountId": accountId,
                "resourceType": resource_type
            })
    ## List Resources 
    ## END

def main():
    doWhateverItNeedsToBeDone = DoWhateverYouSupposeYouShouldDo()
    doWhateverItNeedsToBeDone.doIt()
    # val = input("Enter your value: ") 
    # print(val) 
    # AWS_CREDENTIALS_DIR = "~/.aws/"
    # print("") 
    # print("")
    # print("\033[31m ---> Ops!\033[33m AWS CREDENTIALS NOT FOUND!")
    # print("") 
    # print("") 
    # print("\033[34m ---> \033[33mPlease, configure your AWS Credentials:")
    # print("\033[34m      \033[33m1. Create the folder \033[35m{}\033[0m".format(AWS_CREDENTIALS_DIR))
    # print("\033[34m      \033[33m2. Create the file \033[35m{}credentials\033[33m with the  content:".format(AWS_CREDENTIALS_DIR))
    # print("\033[34m      \033[94m   [default]")
    # print("\033[34m      \033[94m   aws_access_key_id = YOUR_ACCESS_KEY")
    # print("\033[34m      \033[94m   aws_secret_access_key = YOUR_SECRET_KEY")
    # print("\033[34m      \033[33m3. Optionally, create the file \033[35m{}config\033[33m with your default region:".format(AWS_CREDENTIALS_DIR))
    # print("\033[34m      \033[94m   [default]")
    # print("\033[34m      \033[94m   region=us-east-1")
    # print("")
    # print("")


if __name__ == '__main__':
    # python -m pyawscp.JustOneShot
    main()    
        


import datetime
from pyawscp.Utils import Utils, Style

class CookieTempCredentials:

    def __init__(self):
        self.accessKey       = None
        self.secretAccessKey = None
        self.sessionToken    = None
        self.expirationDate  = None
    
    def isExpired(self):
        return self.expirationDate < datetime.datetime.now()

    def humanReadFormatExpirationDate(self):
        return self.expirationDate.strftime("%d-%m-%Y %H:%M:%S")

    def timeToExpire(self):
        if self.expirationDate:
           if self.isExpired():
              return -1
           diff = (self.expirationDate - datetime.datetime.now())
           if diff.seconds > 60:
               min  = int(diff.seconds / 60)
               secs = int(((diff.seconds / 60) - min) * 60)
               return "00:{:02d}:{:02d} minutes".format(min, secs)
           else:   
              return "00:00:{:02d} seconds".format(diff.seconds)
        return 0

    def dateToString(self, dt):
        return dt.strftime("%Y-%m-%d %H:%M:%S")

    def stringToDate(self, strDate):
        return datetime.datetime.strptime(strDate, "%Y-%m-%d %H:%M:%S")


    @staticmethod
    def headerCookieSTS():
        output  = "\n\n" + Style.RESET
        output += "======================"
        output += "\n"
        output += Style.IGREEN + "TEMPORARY SESSION ROLE"
        output += "\n" + Style.RESET
        output += "======================"
        output += "\n"
        return output

   
    @staticmethod
    def invalidCookie():
        output  = CookieTempCredentials.headerCookieSTS()
        output += Style.IRED + "Not found!"
        output += "\n"
        output += Style.GREEN + "There's no active/valid Temporary Session Role"
        output += "\n"
        output += "\n"
        return output

    @staticmethod
    def printCookie(isValid, role, serial, expiration ):
        output = CookieTempCredentials.headerCookieSTS()
        if isValid:
           output += Style.GREEN + "Role.............: " +  Style.IBLUE + role + Style.RESET
           output += "\n"
           output += Style.GREEN + "MFA Serial.......: " +  Style.IBLUE + serial + Style.RESET
           output += "\n"
           output += Style.GREEN + "Time Remaining...: " + Style.IBLUE + str(expiration) + Style.RESET
           output += "\n" 
        else:
           output += Style.IRED + "Not found!"
           output += "\n"
           output += Style.GREEN + "There's no active/valid Temporary Session Role"
           output += "\n"
           output += "\n"
        return output
    



import sys
import os, re
from random import randrange
from os.path import expanduser
from pyawscp.Utils import Utils

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources
from . import templates

class CsvDrawIo:

    def __init__(self, config):
        self.config = config
        self.region = self.config.awsRegion
    

    def buildLink(self, whichLink, resourceId):
        whichLink = whichLink.lstrip().rstrip().upper()

        if "SUBNET" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#subnets:SubnetId={id}" \
                  .format(region=self.region,id=resourceId)
        elif "VPC" == whichLink:
            return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#vpcs:VpcId={id}" \
                  .format(region=self.region,id=resourceId)
        elif "EC2_PRIVATEIP" == whichLink:
           return "https://{region}.console.aws.amazon.com/ec2/v2/home?region={region}#Instances:privateIpAddress={id}" \
                  .format(region=self.region,id=resourceId)
        elif "EC2_INSTANCEID" == whichLink:
           return "https://{region}.console.aws.amazon.com/ec2/v2/home?region={region}#Instances:instanceId={id}" \
                  .format(region=self.region,id=resourceId)        
        elif "ELB" == whichLink:
           return "https://{region}.console.aws.amazon.com/ec2/v2/home?region={region}#LoadBalancers:search={id};sort=state" \
                  .format(region=self.region,id=resourceId)
        elif "SECURITYGROUP" == whichLink:
           return "https://{region}.console.aws.amazon.com/ec2/v2/home?region={region}#SecurityGroup:groupId={id}" \
                  .format(region=self.region,id=resourceId)
        elif "ROUTETABLE" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#RouteTables:search={id};sort=desc:routeTableId" \
                  .format(region=self.region,id=resourceId)
        elif "NACL" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#acls:networkAclId={id}" \
                  .format(region=self.region,id=resourceId)
        elif "TRANSITGATEWAY" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#TransitGateways:search={id};sort=transitGatewayId" \
                  .format(region=self.region,id=resourceId)
        elif "TRANSITGATEWAYROUTETABLE" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#TransitGatewayRouteTables:transitGatewayRouteTableId={id};sort=transitGatewayRouteTableId" \
                  .format(region=self.region,id=resourceId)
        elif "TRANSITGATEWAYATTACHMENT" == whichLink:
            return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#TransitGatewayAttachments:transitGatewayAttachmentId={id};sort=desc:transitGatewayId" \
                  .format(region=self.region,id=resourceId)
        elif "NATGATEWAY" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#NatGateways:natGatewayId={id}" \
                  .format(region=self.region,id=resourceId)
        elif "INTERNETGATEWAY" == whichLink:
           return "https://{region}.console.aws.amazon.com/vpc/home?region={region}#igws:internetGatewayId={id}" \
                  .format(region=self.region,id=resourceId)

        raise ValueError("Not found link {} to be built!", whichLink)

    def openTemplate(self, templateName):
        return pkg_resources.read_text(templates, templateName)

    ##
    ## Route Tables
    ##
    def generateListRouteTables(self, templateName, data):
        template = self.openTemplate(templateName)

        #fileName = os.path.join(expanduser("~"), "listRouteTables.csv") 
        fileName = os.path.join(expanduser("~"), 'listRouteTables_' + Utils.labelTimestamp() + ".csv")
        try:
            os.remove(fileName)
        except OSError:
            pass
        
        csvLines = []
        for routeTable in data["RouteTables"]:
            rtId  = routeTable["routeTableId"]
            vpcId = routeTable["vpcId"] 

            listSubnets  = "\"<ul>"
            if "subnets" in routeTable:
               for subnet in routeTable["subnets"]:
                   subnetLabel = "<a href='{}'>{}</a>".format(self.buildLink("subnet",subnet["subnetId"]),subnet["subnetId"])
                   listSubnets += "<li>{}</li>".format(subnetLabel)
            listSubnets += "</ul>\""

            tableRoutes  = "\"<table width=100% cellpadding=3 cellspacing=0 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
            tableRoutes += "<tr style='border-bottom:1px black solid'><td colspan='4' style='background:#2F4F4F;color:white;text-align:center;font-style:italic;font-weight:bolder;text-transform:uppercase;'>Routes</td></tr>"
            tableRoutes += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>Destination</td>" + \
                               "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Target</td>" + \
                               "<td align='center' style='border-top:1px black solid'>Status</td>" + \
                            "</tr>"

            colorEven = "#D3D3D3"
            colorOdd  = "#FFFFFF"
            idx       = 0
            for route in routeTable["routes"]:
                lineBackColor  = colorEven if idx % 2 == 0 else colorOdd

                if route["cidrblock"] == "0.0.0.0/0" and route["target"].startswith("igw-"):
                   colorLine     = "red;font-weigth:bolder;"
                   lineBackColor = "yellow"
                else:
                   colorLine = "blue"
                
                linkTarget = route["target"]
                if "tgw-" in route["target"]:
                    linkTarget = "<a href='{}'>{}</a>".format(self.buildLink("TRANSITGATEWAY",route["target"]),route["target"])
                elif "i-" in route["target"]:
                    linkTarget = "<a href='{}'>{}</a>".format(self.buildLink("EC2_INSTANCEID",route["target"]),route["target"])
                elif "nat-" in route["target"]:
                    linkTarget = "<a href='{}'>{}</a>".format(self.buildLink("NATGATEWAY",route["target"]),route["target"])
                elif "igw-" in route["target"]:
                    linkTarget = "<a href='{}'>{}</a>".format(self.buildLink("INTERNETGATEWAY",route["target"]),route["target"])

                lineTableRoute = "<tr style='background:{linebackcolor};'><td style='border-right:1px black solid;border-top:1px black solid;color:{colorLine}'>{}</td>" + \
                                   "<td style='border-right:1px black solid;border-top:1px black solid;color:{colorLine}'>{}</td>" + \
                                   "<td style='border-top:1px black solid;color:{colorLine}'>{}</td>" + \
                                 "</tr>"
                tableRoutes += lineTableRoute.format(route["cidrblock"], linkTarget, route["state"], linebackcolor=lineBackColor, colorLine=colorLine)
                idx += 1
            tableRoutes += "</table><br>\""

            labelVpcId = "\"<a href='{}'>{}</a>\"".format(self.buildLink("VPC",vpcId),vpcId)

            rtIdStamp = "&nbsp;&nbsp;&nbsp;&nbsp;<div style='border-color:black;border-width:1px;border-style:solid;border-radius:3px;display:inline;color:white;background:blue;font-size:16px'>&nbsp;&nbsp;" + \
                        rtId[-5:] + \
                        "&nbsp;&nbsp;</div>"
            labelRtId = "\"<b>"+rtId+"</b>" + rtIdStamp + "\""

            line = "{},{},{},{},{},{}".format(labelRtId,labelVpcId,listSubnets,tableRoutes, \
                                         "#E6E6E6", \
                                         self.buildLink("routetable",rtId))
            csvLines.append(line)


        with open(fileName, 'w') as csvFile:
             csvFile.write(template + '\n')
             for line in csvLines:
                 csvFile.write(line + '\n')
        return fileName

    ##
    ## Security Groups
    ##    
    def generateListSg(self, templateName, header, data):
        template = self.openTemplate(templateName)

        #fileName = os.path.join(expanduser("~"), "listSg.csv") 
        fileName = os.path.join(expanduser("~"), 'listSg_' + Utils.labelTimestamp() + ".csv")
        try:
            os.remove(fileName)
        except OSError:
            pass
        
        csvLines = []
        for securityGroup in data["sgs"]:
            sgId  = securityGroup["sg"]["id"] 
            vpcId = securityGroup["sg"]["vpcId"] 

            listNetWorkInterfaces  = "\"<ul>"
            if "networkInterfaces" in securityGroup["sg"]:
               for netIntf in securityGroup["sg"]["networkInterfaces"]:

                   if netIntf["Description"] == "":
                      descriptionResource = "<a href='{}'>EC2</a>".format(self.buildLink("EC2_PRIVATEIP",netIntf["PrivateIpAddress"]))
                   elif "ELB " in netIntf["Description"]:
                      elbName = re.sub(r'^ELB\s',"",netIntf["Description"])
                      descriptionResource = "<a href='{}'>{}</a>".format(self.buildLink("ELB",elbName), netIntf["Description"])
                   else:
                      descriptionResource = netIntf["Description"]

                   listNetWorkInterfaces += "<li>{} - {} ({})</li>".format(netIntf["NetworkInterfaceId"], descriptionResource, netIntf["PrivateIpAddress"])
            listNetWorkInterfaces += "</ul>\""
        
            colorEven = "#D3D3D3"
            colorOdd  = "#FFFFFF"
            idx       = 0

            tableIngress  = "\"<table width=100% cellspacing=0 cellpadding=3 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
            tableIngress += "<tr style='border-bottom:1px black solid'><td colspan='4' style='background:#2F4F4F;color:white;text-align:center;font-style:italic;font-weight:bolder;text-transform:uppercase;'>Inbound Rules</td></tr>"
            tableIngress += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>Protocol</td>" + \
                               "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Port Range</td>" + \
                               "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Source</td>" + \
                               "<td align='center' style='border-top:1px black solid'>Description</td>" + \
                            "</tr>"

            if securityGroup["sg"]["ipPermissions"]:
               for ingress in securityGroup["sg"]["ipPermissions"]:
                   for ipRange in ingress["ipRanges"]:
                     
                       protocol = "All" if ingress["ipProtocol"].rstrip().lstrip() == "-1" else str(ingress["ipProtocol"]).upper()
   
                       portRange = str(ingress["fromPort"])
                       if ingress["fromPort"] != ingress["toPort"]:
                          portRange += ("-" + str(ingress["toPort"]))
   
                       lineBackColor  = colorEven if idx % 2 == 0 else colorOdd
                       lineTableIngress = "<tr style='background:{linebackcolor};'><td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                            "<td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                            "<td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                            "<td style='border-top:1px black solid'>{}</td>" + \
                                          "</tr>"
                       
                       tableIngress += lineTableIngress \
                                       .format(protocol, \
                                               portRange, \
                                               ipRange["cidrIp"], \
                                               ipRange["description"], \
                                               linebackcolor=lineBackColor)
                       idx+=1
            tableIngress += "</table>\""

            idx = 0

            tableEgress  = "\"<table width=100% cellspacing=0 cellpadding=3 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
            tableEgress += "<tr style='border-bottom:1px black solid'><td colspan='4' style='background:#2F4F4F;color:white;text-align:center;font-style:italic;font-weight:bolder;text-transform:uppercase;'>Outbound Rules</td></tr>"
            tableEgress += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>Protocol</td>" + \
                              "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Port Range</td>" + \
                              "<td align='center' style='border-top:1px black solid'>Destination</td>" + \
                           "</tr>"
            for egress in securityGroup["sg"]["ipPermissionsEgress"]:

                protocol = "All" if egress["ipProtocol"].rstrip().lstrip() == "-1" else str(egress["ipProtocol"]).upper()

                for ipRange in egress["ipRanges"]:

                    lineBackColor  = colorEven if idx % 2 == 0 else colorOdd
                    lineTableEgress = "<tr style='background:{linebackcolor};'><td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                         "<td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                         "<td style='border-top:1px black solid'>{}</td>" + \
                                       "</tr>"
                    tableEgress += lineTableEgress.format(protocol, \
                                                          ipRange["cidrIp"], \
                                                          ipRange["description"], \
                                                          linebackcolor=lineBackColor)
                    idx+=1
            tableEgress += "</table>\""

            line = "{},{},{},{},{},{},{}".format(sgId,vpcId,listNetWorkInterfaces,tableIngress,tableEgress, \
                                         "#DFF2E2", \
                                         self.buildLink("SECURITYGROUP",sgId))
            csvLines.append(line)


        with open(fileName, 'w') as csvFile:
             csvFile.write(template + '\n')
             for line in csvLines:
                 csvFile.write(line + '\n')
        return fileName

    ##
    ## NACLs
    ##
    def generateListNacls(self, templateName, header, content):
        template = self.openTemplate(templateName)

        #fileName = os.path.join(expanduser("~"), "listNacls.csv") 
        fileName = os.path.join(expanduser("~"), 'listNacls_' + Utils.labelTimestamp() + ".csv")
        try:
            os.remove(fileName)
        except OSError:
            pass

        currentnaclId = None
        currentVpcId  = None
        csvLines      = []
        
        for row in content:
            vpcId    = row[1]
            naclId   = row[2]
            subnetId = row[3]
            entries  = row[4] if len(row) > 4 else ""

            if not currentnaclId or currentnaclId != naclId:
               if currentnaclId:
                  labelCurrentVpc = "\"<a href='{}'>{}</a>\"".format(self.buildLink("VPC",currentVpcId),currentVpcId) 
                  csvLines.append( self.createNaclLine(currentnaclId, indexRule, subnets, rules, labelCurrentVpc) )
                  
               currentnaclId      = naclId
               currentVpcId       = vpcId
               indexRule          = 0
               rules              = []
               subnets            = []
        
            if subnetId.rstrip().lstrip() != "":
               subnets.append(subnetId)

            # Rules
            entrySplit = entries.split("|")
            for idx, r in enumerate(entrySplit):
                if idx == 0:
                   cidrblock =  self.cleanRuleField(r)
                elif idx == 1:
                   type =  self.cleanRuleField(r)
                elif idx == 2:
                   state =  self.cleanRuleField(r)
                elif idx == 3:
                   number =  int(self.cleanRuleField(r))
                elif idx == 4:
                   protocol =  self.cleanRuleField(r) 
                   indexRule += 1
                   record = {"index": indexRule, "cidrblock": cidrblock,"type": type, "state": state, "number": number, "protocol": protocol}
                   rules.append(record)

        if currentnaclId:
           labelCurrentVpc = "\"<a href='{}'>{}</a>\"".format(self.buildLink("VPC",currentVpcId),currentVpcId)
           csvLines.append( self.createNaclLine(currentnaclId, indexRule, subnets, rules, labelCurrentVpc) )

        with open(fileName, 'w') as csvFile:
             csvFile.write(template + '\n')
             for line in csvLines:
                 csvFile.write(line + '\n')
        return fileName   
    def createNaclLine(self, currentnaclId, indexRule, subnets, rules, vpcId):
        subnetTable = "\"<table cellspacing=0 cellpadding=0>"
        for s in subnets:
            s = re.sub('\(\d*\)','',s)
            s = s.rstrip().lstrip()
            labelSubnet = "<a href='{}'>{}</a>".format(self.buildLink("SUBNET",s),s)
            subnetTable += "<tr><td>&nbsp;&nbsp;-&nbsp;" + labelSubnet + "</td></tr>"
        subnetTable += "</table>\""

        colorEven = "#D3D3D3"
        colorOdd  = "#FFFFFF"
        idx       = 0
        
        rulesTableInbound = "\"<table width=100% cellpadding=3 cellspacing=0 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
        rulesTableInbound += "<tr style='border-bottom:1px black solid'><td colspan='4' style='background:#2F4F4F;color:white;text-align:center;font-style:italic;font-weight:bolder;text-transform:uppercase;'>Inbound Rules</td></tr>"
        rulesTableInbound += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>Rules Number</td>" + \
                                 "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Protocol</td>" + \
                                 "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Source</td>" + \
                                 "<td align='center' style='border-top:1px black solid'>Allow/Deny</td>" + \
                            "</tr>"
        
        
        for e in rules:
            if e["type"] == "Egress":
               lineBackColor  = colorEven if idx % 2 == 0 else colorOdd
               stateColor     = "red" if e["state"].lower() == "deny" else "blue"
               rulesTableInbound += "<tr style='border-bottom:1px black solid;background:{linebackcolor}'>".format(linebackcolor=lineBackColor)
               rulesTableInbound += "  <td style='border-right:1px black solid;border-top:1px black solid;color:{}'>".format(stateColor) + ("*" if e["number"] == 32767 else str(e["number"])) + "</td>"
               rulesTableInbound += "  <td style='border-right:1px black solid;border-top:1px black solid;color:{}'>".format(stateColor) + ("All" if e["protocol"] == "-1" else e["protocol"]) + "</td>"
               rulesTableInbound += "  <td style='border-right:1px black solid;border-top:1px black solid;color:{}'>".format(stateColor) + e["cidrblock"] + "</td>"
               rulesTableInbound += "  <td style='border-top:1px black solid;color:{}'>".format(stateColor) + e["state"].capitalize() + "</td>"
               rulesTableInbound += "</tr>"
               idx+=1
        rulesTableInbound   += "</table>\""

        idx = 0

        rulesTableOutbound = "\"<table width=100% cellpadding=3 cellspacing=0 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
        rulesTableOutbound += "<tr style='border-bottom:1px black solid'><td colspan='4' style='background:#2F4F4F;color:white;text-align:center;font-style:italic;font-weight:bolder;text-transform:uppercase;'>Outbound Rules</td></tr>"
        rulesTableOutbound += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>Rules Number</td>" + \
                                 "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Protocol</td>" + \
                                 "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Source</td>" + \
                                 "<td align='center' style='border-top:1px black solid'>Allow/Deny</td>" + \
                            "</tr>"
        for e in rules:
            if e["type"] != "Egress":
               lineBackColor  = colorEven if idx % 2 == 0 else colorOdd
               stateColor     = "red" if e["state"].lower() == "deny" else "blue"
               rulesTableOutbound += "<tr style='border-bottom:1px black solid;background:{linebackcolor}'>".format(linebackcolor=lineBackColor)
               rulesTableOutbound += "  <td style='border-right:1px black solid;border-top:1px black solid;color:{}'>".format(stateColor) + ("*" if e["number"] == 32767 else str(e["number"])) + "</td>"
               rulesTableOutbound += "  <td style='border-right:1px black solid;border-top:1px black solid;color:{}'>".format(stateColor) + ("All" if e["protocol"] == "-1" else e["protocol"]) + "</td>"
               rulesTableOutbound += "  <td style='border-right:1px black solid;border-top:1px black solid;color:{}'>".format(stateColor) + e["cidrblock"] + "</td>"
               rulesTableOutbound += "  <td style='border-top:1px black solid;color:{}'>".format(stateColor) + e["state"].capitalize() + "</td>"
               rulesTableOutbound += "</tr>"
               idx+=1
        rulesTableOutbound   += "</table>\""

        naclid5digits = currentnaclId[-5:]
        naclidStamp   = "&nbsp;&nbsp;&nbsp;&nbsp;<div style='border-color:black;border-width:1px;border-style:solid;border-radius:3px;display:inline;color:white;background:blue;font-size:16px'>&nbsp;&nbsp;" + naclid5digits + "&nbsp;&nbsp;</div>"
        finalLine     = "\"<b>"+currentnaclId+"</b>" + naclidStamp + "\"" \
                    + "," \
                    + vpcId \
                    + "," + \
                    subnetTable   \
                    + "," + \
                    rulesTableInbound \
                    + "," + \
                    rulesTableOutbound   \
                    + "," + \
                    "#dae8fc" \
                    + "," + \
                    self.buildLink("NACL",currentnaclId)
        return finalLine
    def cleanRuleField(self, field):
        return re.sub('\(\d*\)','',field.rstrip().lstrip())

    
    


from diagrams import Diagram, Edge, Node, Cluster
from diagrams.custom import Custom
from diagrams.aws.compute import EC2
from diagrams.aws.database import RDS
from diagrams.aws.network import ELB
from diagrams.aws.network import VPC
from diagrams.aws.network import PublicSubnet
from diagrams.aws.network import PrivateSubnet


shapes         = ["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15"]
num_shapes     = len(shapes)
shapes_per_row = 5
num_of_rows    = int(num_shapes / shapes_per_row) + (num_shapes % shapes_per_row > 0)

with Diagram("Diagram", show=False) as test2:
    with Cluster("vpc"):
        with Cluster("subnet"):
            for row in range(num_of_rows)[::-1]:
                items_in_row = shapes_per_row - (row+1) * shapes_per_row // num_shapes
                shapes_i = row * shapes_per_row
                node_list = ['EC2(\'' + shapes[shapes_i+item_num] + '\') - Edge(penwidth="0.0")'
                    for item_num in range(items_in_row)[:-1]
                ] + ['EC2(\'' + shapes[shapes_i+items_in_row-1] + '\')']

                node_row = "-".join(node_list)
                eval(node_row)

test2




import boto3
import os
import re
import sys
import datetime
from datetime import timedelta
from os.path import expanduser
from pyawscp.Utils import Utils, Style
from pyawscp.CookieTempCredentials import CookieTempCredentials
import configparser

class Config:
 
    TMP_CREDENTIALS_FILE = expanduser("~") + "/.pyawscp/tmp.bin"

    def __init__(self):
        self.awsProfile               = ""
        self.awsRegion                = ""
        self.awsTags                  = {}
        self.command                  = ""
        self.commandArguments         = ""
        self.tableLineSeparator       = True
        self.interactive              = False
        self.printResults             = False
        self.uploadChunkSizeMultipart = 10
        self.uploadThresholdMultipart = 100
        self.assumeRole               = None
        self.mfaSerial                = None
        self.mfaPassCode              = None

    def awsTagsToFilter(self):
        if self.awsTags and len(self.awsTags) > 0:
            return True
        return False

    # AssumeRole
    def assumeRoleSession(self, roleArn, mfaToken):
        tempCredentials = boto3.client('sts').assume_role(
            RoleArn=roleArn,
            RoleSessionName="mysession",
            DurationSeconds=3600,
            SerialNumber=self.mfaSerial,
            TokenCode=mfaToken
        )
        return tempCredentials


    def evalCmdToDate(self, cmdDate):
        txt = str(cmdDate)

        ## Method 1
        # nums = re.search('\d{4}.*,',txt).group(0).split(",")
        # print(nums)
        # year  = int(nums[0])
        # month = int(nums[1])
        # day   = int(nums[2])
        # hour  = int(nums[3])
        # minu  = int(nums[4])
        # seco  = int(nums[5])
        # expiration = datetime.datetime(year, month, day, hour, minu, seco)

        # Method 2
        dt = eval(re.sub(",\s{1,}tzinfo=tzutc\(\)","",txt), {'datetime': datetime}, dict())
        return dt
        # Add the duration (1h)
        return dt + timedelta(hours=1)

    def dateToString(self, dt):
        return dt.strftime("%Y-%m-%d %H:%M:%S")

    def stringToDate(self, strDate):
        return datetime.datetime.strptime(strDate, "%Y-%m-%d %H:%M:%S")

    def cookieTempCredentials(self):
        return self.loadCookieTempCredentials()

    def isValidCookieTempCredentials(self):
        cookie = self.loadCookieTempCredentials()
        if cookie and not cookie.isExpired():
           return True
        return False

    def saveCookieTempCredentials(self, tempCredentials):
        file = configparser.ConfigParser(allow_no_value=True) 
        file.add_section("COOKIE")
        file.set("COOKIE","access-key",        tempCredentials['Credentials']['AccessKeyId'])
        file.set("COOKIE","secret-access-key", tempCredentials['Credentials']['SecretAccessKey'])
        file.set("COOKIE","session-token",     tempCredentials['Credentials']['SessionToken'])

        expirationDate = tempCredentials['Credentials']['Expiration']
        # Add the duration (1h)
        expirationDate = expirationDate + timedelta(hours=1)
        file.set("COOKIE","expiration", self.dateToString(expirationDate))

        with open(self.TMP_CREDENTIALS_FILE,'w') as configfile:
             file.write(configfile)

    def loadCookieTempCredentials(self):
        if os.path.exists(self.TMP_CREDENTIALS_FILE):
           file = configparser.ConfigParser()
           file.read(self.TMP_CREDENTIALS_FILE) 
           cookie = CookieTempCredentials()
           cookie.accessKey       = file["COOKIE"]["access-key"]
           cookie.secretAccessKey = file["COOKIE"]["secret-access-key"]
           cookie.sessionToken    = file["COOKIE"]["session-token"]  
           cookie.expirationDate  = self.stringToDate(file["COOKIE"]["expiration"])
           return cookie
        return None

    def credentialsExpired(self, expirationDate):
        return expirationDate < datetime.datetime.now()

    def createCookieRoleSession(self):
        if not self.mfaSerial:
           output = "\n\n\n"
           output += Style.RED  + "=".ljust(108,"=") +"\n"
           output += Style.BLUE + " The Assume Role " + Style.RESET + "\"" + Style.GREEN + self.assumeRole + Style.RESET + "\"" + Style.BLUE + " is configured for the environment" + Style.RESET + "\n" 
           output += Style.BLUE + " A MFA Serial is needed in order to establish a valida session, command:  " + Style.GREEN+ " set mfa-serial {arn:aws:iam::...}" + Style.RESET + "\n" 
           output += Style.RED  + "=".ljust(108,"=") +"\n\n"
           print(output)
           sys.exit()
        
        if not self.mfaPassCode:
           output = "\n\n\n"
           output += Style.RED  + "=".ljust(97,"=") +"\n"
           output += Style.BLUE + " The Assume Role " + Style.RESET + "\"" + Style.GREEN + self.assumeRole + Style.RESET + "\"" + Style.BLUE + " is configured for the environment" + Style.RESET + "\n" 
           output += Style.BLUE + " Please, set a valid MFA PassCode to this Session using the command:  " + Style.GREEN+ " set mfa {passcode}" + Style.RESET + "\n" 
           output += Style.RED  + "=".ljust(97,"=") +"\n\n"
           print(output)
           sys.exit()

        tempCredentials = self.assumeRoleSession(self.assumeRole, self.mfaPassCode) 
        self.saveCookieTempCredentials(tempCredentials)

        cookie = CookieTempCredentials()
        cookie.accessKey       = tempCredentials['Credentials']['AccessKeyId']
        cookie.secretAccessKey = tempCredentials['Credentials']['SecretAccessKey']
        cookie.sessionToken    = tempCredentials['Credentials']['SessionToken']
        cookie.expirationDate  = self.dateToString(tempCredentials['Credentials']['Expiration'])
        return cookie

    def deleteCookieTemporaryCredentials(self):
        if os.path.exists(self.TMP_CREDENTIALS_FILE):
           os.remove(self.TMP_CREDENTIALS_FILE)
        
        
    def botoSession(self):
        if self.assumeRole:
           isNewCookie = False
           cookie      = self.loadCookieTempCredentials()
           if not cookie or cookie.isExpired():
              cookie      = self.createCookieRoleSession()
              isNewCookie = True

           newSession = None 
           if not self.awsRegion:
              newSession = boto3.session.Session(
                  aws_access_key_id=cookie.accessKey,
                  aws_secret_access_key=cookie.secretAccessKey,
                  aws_session_token=cookie.sessionToken
              )
           else:   
              newSession = boto3.session.Session(
                  aws_access_key_id=cookie.accessKey,
                  aws_secret_access_key=cookie.secretAccessKey,
                  aws_session_token=cookie.sessionToken,
                  region_name=self.awsRegion
              )
           return newSession   
        return self.botoSessionCredentialsOwner()

    def startSessionWithTheNewCookie(self):
        self.botoSession().client('ec2').describe_vpcs() # doesnt matter the result, as long as the connection to create the Cookie is established (to not waste the given MFA password)
        cookie = self.loadCookieTempCredentials()
        output = CookieTempCredentials.printCookie(self.isValidCookieTempCredentials(), \
                                                   self.assumeRole, \
                                                   self.mfaPassCode, \
                                                   cookie.timeToExpire())
        return output

    def botoSessionCredentialsOwner(self):
        if not self.awsProfile and not self.awsRegion:
           session = boto3.Session()
        elif not self.awsProfile and     self.awsRegion:    
           session = boto3.Session(region_name=self.awsRegion)
        elif     self.awsProfile and not self.awsRegion:   
           session = boto3.Session(profile_name=self.awsProfile)
        else:
           session = boto3.Session(profile_name=self.awsProfile,region_name=self.awsRegion) 
        return session

    def isThereAwsCredentials(self):
        AWS_CREDENTIALS_DIR  = expanduser("~") + "/.aws/"
        AWS_CREDENTIALS_FILE = AWS_CREDENTIALS_DIR + "credentials"
        if not os.path.exists(AWS_CREDENTIALS_DIR) or not os.path.exists(AWS_CREDENTIALS_FILE):
           print("") 
           print("")
           print("\033[31m ---> Ops!\033[33m AWS CREDENTIALS NOT FOUND!")
           print("") 
           print("") 
           print("\033[34m ---> \033[33mPlease, configure your AWS Credentials:")
           print("\033[34m      \033[33m1. Create the folder \033[35m{}\033[0m".format(AWS_CREDENTIALS_DIR))
           print("\033[34m      \033[33m2. Create the file \033[35m{}credentials\033[33m with the  content:".format(AWS_CREDENTIALS_DIR))
           print("\033[34m      \033[94m   [default]")
           print("\033[34m      \033[94m   aws_access_key_id = YOUR_ACCESS_KEY")
           print("\033[34m      \033[94m   aws_secret_access_key = YOUR_SECRET_KEY")
           print("\033[34m      \033[33m3. Optionally, create the file \033[35m{}config\033[33m with your default region:".format(AWS_CREDENTIALS_DIR))
           print("\033[34m      \033[94m   [default]")
           print("\033[34m      \033[94m   region=us-east-1")
           print("\033[0m")
           print("")
           return False
        return True


    




# coding=utf-8

import sys


class Test:

    def __init__(self):
        self.my_function(a=12, _vpcid=123456)

    def my_function(self, **args):
        print(args["_vpcid"])

## Troubleshoot
## in WSL (Bash), in case the Enter is resulting a ^M   use this command "stty sane" before call Python
def main():
    test = Test()
    # val = input("Enter your value: ") 
    # print(val) 
    # AWS_CREDENTIALS_DIR = "~/.aws/"
    # print("") 
    # print("")
    # print("\033[31m ---> Ops!\033[33m AWS CREDENTIALS NOT FOUND!")
    # print("") 
    # print("") 
    # print("\033[34m ---> \033[33mPlease, configure your AWS Credentials:")
    # print("\033[34m      \033[33m1. Create the folder \033[35m{}\033[0m".format(AWS_CREDENTIALS_DIR))
    # print("\033[34m      \033[33m2. Create the file \033[35m{}credentials\033[33m with the  content:".format(AWS_CREDENTIALS_DIR))
    # print("\033[34m      \033[94m   [default]")
    # print("\033[34m      \033[94m   aws_access_key_id = YOUR_ACCESS_KEY")
    # print("\033[34m      \033[94m   aws_secret_access_key = YOUR_SECRET_KEY")
    # print("\033[34m      \033[33m3. Optionally, create the file \033[35m{}config\033[33m with your default region:".format(AWS_CREDENTIALS_DIR))
    # print("\033[34m      \033[94m   [default]")
    # print("\033[34m      \033[94m   region=us-east-1")
    # print("")
    # print("")

if __name__ == '__main__':
    main()    
        


import sys
import os, re
from random import randrange
from os.path import expanduser
from pyawscp.Utils import Utils
from pyawscp.CsvDrawIo import CsvDrawIo
from pygments import highlight, lexers, formatters

class CsvDrawIoNetworking(CsvDrawIo):

    DEFAULT_WIDTH  = 50
    DEFAULT_HEIGHT = 50

    VPC_COLS        = 2
    VPC_XCOORD      = 20
    VPC_YCOORD      = 20
    VPC_WIDTH       = 1000
    VPC_HEIGHT      = 550
    VPC_GAP_ROWS    = 100
    VPC_GAP_COLS    = 80
    VPC_BOTTOM      = int(VPC_HEIGHT - 48) # Position in Px of the bottom of the Subnets boxes

    SUBNET_COLS     = 4
    SUBNET_XMARGIN  = 50
    SUBNET_YMARGIN  = 40
    SUBNET_GAP_ROWS = 20
    SUBNET_GAP_COLS = 30
    SUBNET_WIDTH    = 210
    SUBNET_HEIGHT   = 180
    SUBNET_BOTTOM   = int(SUBNET_HEIGHT - 65)

    ROUTETABLE_WIDTH   = 60
    ROUTETABLE_HEIGHT  = 60
    ROUTETABLE_XMARGIN = 40
    ROUTETABLE_YMARGIN = (int(SUBNET_HEIGHT / 4)) + 8

    NACLS_WIDTH   = 60
    NACLS_HEIGHT  = 60
    NACLS_XMARGIN = 110
    NACLS_YMARGIN = (int(SUBNET_HEIGHT / 4)) + 8

    TGWS_XCOORD      = 80
    TGWS_YCOORD      = 5
    TGWS_GAP_COLS    = 400
    TGWS_GAP_ROWS    = 80
    TGWS_HEIGHT      = 60
    TGWS_WIDTH       = 120
    TGWS_HEIGHT_ICON = 60
    TGWS_WIDTH_ICON  = 60
    TGWS_COLS        = 6

    TGWATTACHMENTS_XCOORD   = -800
    TGWATTACHMENTS_YCOORD   = -10
    TGWATTACHMENTS_GAP_COLS = 20
    TGWATTACHMENTS_GAP_ROWS = 100
    TGWATTACHMENTS_WIDTH    = 700
    TGWATTACHMENTS_COLS     = 1

    TGWROUTETABLE_XCOORD   = 50
    TGWROUTETABLE_YCOORD   = -400
    TGWROUTETABLE_WIDTH    = 650
    TGWROUTETABLE_GAP_COLS = 50

    AZ_WIDTH       = 1050
    AZ_HEIGHT      = 250

    def __init__(self, config):
        super().__init__(config)
        self.graph = Graph()
    
    def adjustments(self, data, addRouteTables, addNacls, addTgws, addRtTgws):
        totalAzs = []
        # WIDTH VPC by Qtde Subnets per AZs in VPC
        qtdeSubnetsByVpcAz = {}
        for vpc in data["vpcs"]:
            for subnet in vpc["subnets"]:
                # Azs by VPC
                groupByKey = "{}_{}".format(vpc["vpcId"],subnet["az"])
                if groupByKey in qtdeSubnetsByVpcAz:
                   qtdeSubnetsByVpcAz[groupByKey]["total"] = qtdeSubnetsByVpcAz[groupByKey]["total"] + 1
                else:
                   qtdeSubnetsByVpcAz[groupByKey] = {
                       "total": 1
                   }
                
                # total Azs
                if not subnet["az"] in totalAzs:
                   totalAzs.append(subnet["az"])
                
        qtdeSubnetsByAz = 0
        for t in qtdeSubnetsByVpcAz:
            if qtdeSubnetsByVpcAz[t]["total"] > qtdeSubnetsByAz:
               qtdeSubnetsByAz = qtdeSubnetsByVpcAz[t]["total"]
        if qtdeSubnetsByAz > 4:
           self.SUBNET_COLS = 4
        else:
           self.SUBNET_COLS = qtdeSubnetsByAz
        self.VPC_WIDTH  = (self.SUBNET_COLS * (self.SUBNET_WIDTH  + self.SUBNET_GAP_COLS)) + self.SUBNET_XMARGIN

        # Calculate Size of the AZ (Rows to accomodate the most "crowed" VPC)
        biggerQtdeAzs = 0
        for i in qtdeSubnetsByVpcAz:
            if qtdeSubnetsByVpcAz[i]["total"] > biggerQtdeAzs:
               biggerQtdeAzs = qtdeSubnetsByVpcAz[i]["total"]
        rowsForAz  = int(biggerQtdeAzs / 4)
        rowsForAz += (0 if ( (biggerQtdeAzs / 4) == int(biggerQtdeAzs / 4) ) else 1)
        self.AZ_HEIGHT = self.AZ_HEIGHT * (rowsForAz if rowsForAz > 0 else 1)

        if addTgws:
           self.VPC_YCOORD = 150
        
        # Calculate Size of the VPC
        self.VPC_HEIGHT = (self.AZ_HEIGHT + 25) * len(totalAzs)
        self.VPC_BOTTOM = int(self.VPC_HEIGHT - 45)

    ##
    ## Generate the AWS Networking Diagram
    ##
    def generateNetworkingGraph(self, templateName, data, addRouteTables, addNacls, addTgws, addRtTgws):
        self.adjustments(data, addRouteTables, addNacls, addTgws, addRtTgws)
        template = self.openTemplate(templateName)

        fileName = os.path.join(expanduser("~"), 'networkingDiagram_' + Utils.labelTimestamp() + ".csv")
        try:
           os.remove(fileName)
        except OSError:
           pass
        
        # The order matters!!! don't change it!
        self.addVPCs(data)
        self.addAZs(data)
        self.addSubnets(data)
        if addRouteTables:
           self.addRouteTables(data)
        if addNacls:
           self.addNacls(data)
        if addTgws:
           self.addTgws(data)
        if addRtTgws:
           self.addRtTgws(data) 

        linesForClipboard = ""

        # Save File
        with open(fileName, 'w') as csvFile:
             csvFile.write(template + '\n')
             linesForClipboard += (template + '\n')
             for line in self.graph.csvLines:
                 csvFile.write(line + '\n')

        # Copy to Clipboard
        for l in self.graph.csvLines:
            linesForClipboard += (l + "\n")
        Utils.addToClipboard(linesForClipboard)

        return fileName
    
    def addRtTgws(self, data):
        xcoord      = self.TGWROUTETABLE_XCOORD
        ycoord      = self.TGWROUTETABLE_YCOORD
        widthTable  = self.TGWROUTETABLE_WIDTH
        gapCol      = self.TGWROUTETABLE_GAP_COLS

        colorEven   = "#D3D3D3"
        colorOdd    = "#FFFFFF"
        totalTables = 0

        for tgw in data["tgws"]:
            refs = tgw["transitGatewayId"]

            for rt in tgw["routeTables"]:
                """ colorful_json = highlight(Utils.dictToJson(rt), lexers.JsonLexer(), formatters.TerminalFormatter())
                print(" {} ".format(rt["routeTableId"]))
                print(colorful_json)
                print("-".ljust(80,"-")) """

                tgwRouteTableUrl  = "<a style='text-decoration:none;color:yellow;' href='{}'>{}</a>".format(self.buildLink("transitgatewayroutetable", \
                                                                 rt["routeTableId"]), rt["routeTableId"])
                totalTables      += 1
                idx               = 0
                htmlTable  = "\"<table width=100% cellspacing=0 cellpadding=3 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
                htmlTable += "<tr style='border-bottom:1px black solid'><td colspan='4' style='background:#2F4F4F;color:white;'>" + \
                             "<span style='text-align:center;font-style:italic;'>Transit Gateway Route Table</span>" + \
                             "<br>{}</br></td></tr>".format(tgwRouteTableUrl)
                htmlTable += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>CIDR</td>" + \
                                "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Attachment</td>" + \
                                "<td align='center' style='border-top:1px black solid'>Resource Type</td>" + \
                             "</tr>"

                totalRoutes = 0
                for route in rt["routes"]:
                    for routeAtt in route["attachments"]:
                        totalRoutes += 1

                        lineBackColor = colorEven if idx % 2 == 0 else colorOdd
                        lineTable     = "<tr style='background:{linebackcolor};'><td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                            "<td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                            "<td style='border-top:1px black solid'>{}</td>" + \
                                        "</tr>"
                        
                        attachmentLabel = "<a href='{}'>{}</a>".format(self.buildLink("transitgatewayattachment", routeAtt["transitGatewayAttachmentId"]),routeAtt["transitGatewayAttachmentId"])
                        resourceLabel   = routeAtt["resourceId"]
                        if routeAtt["resourceType"] == "vpc":
                           resourceLabel = "<a href='{}'>{}</a>".format(self.buildLink("vpc", routeAtt["resourceId"]),routeAtt["resourceId"]) 
                        htmlTable += lineTable \
                                    .format(route["destinationCidrBlock"], \
                                     "{} | {}".format(attachmentLabel,resourceLabel), \
                                     routeAtt["resourceType"], \
                                     linebackcolor=lineBackColor)
                        idx+=1

                htmlTable  += "</table>\""
                heightTable = idx * 38
                xcoord      = (widthTable * (totalTables-1)) + (gapCol if totalTables > 1 else 0)
                self.graph.csvLines.append(self.createLineTgwRouteTable(rt, htmlTable,  xcoord, ycoord, widthTable, heightTable, refs))

    def addTgws(self, data):
        xcoord = self.TGWS_XCOORD
        ycoord = self.TGWS_YCOORD

        #TGWS
        colorEven = "#D3D3D3"
        colorOdd  = "#FFFFFF"

        gapCol    = self.TGWS_GAP_COLS
        gapRow    = self.TGWS_GAP_ROWS
        cols      = 1
        colsAttch = 1
        totalHeightAttachments = 0
        for tgw in data["tgws"]:
            idx               = 0
            refs              = "\""
            attachmentsTable  = "\"<table width=100% cellspacing=0 cellpadding=3 style='background:white;border:1px black solid;font-family: Courier New, Courier, monospace;font-size: 12px;'>"
            attachmentsTable += "<tr style='background:#696969;color:white;'><td align='center' style='border-right:1px black solid;border-top:1px black solid'>TGW Attachment</td>" + \
                                  "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>TGW Route Table</td>" + \
                                  "<td align='center' style='border-right:1px black solid;border-top:1px black solid'>Resource</td>" + \
                                  "<td align='center' style='border-top:1px black solid'>Resource Owner</td>" + \
                               "</tr>"

            totalAttachments   = 0
            for attachment in tgw["attachments"]:
                totalAttachments += 1

                lineBackColor        = colorEven if idx % 2 == 0 else colorOdd
                lineAttachmentsTable = "<tr style='background:{linebackcolor};'><td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                         "<td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                         "<td style='border-right:1px black solid;border-top:1px black solid'>{}</td>" + \
                                         "<td style='border-top:1px black solid'>{}</td>" + \
                                       "</tr>"
                if refs != "\"":
                   refs += ","
                refs += attachment["resourceId"]

                tgwRouteTableUrl = "<a href='{}'>{}</a>".format(self.buildLink("transitgatewayroutetable", attachment["association"]["transitGatewayRouteTableId"]),attachment["association"]["transitGatewayRouteTableId"])
                attachmentsTable += lineAttachmentsTable \
                                    .format(attachment["transitGatewayAttachmentId"], \
                                            tgwRouteTableUrl, \
                                            attachment["resourceId"], \
                                            attachment["resourceOwnerId"], \
                                            linebackcolor=lineBackColor)
                idx+=1

            attachmentsTable += "</table>\""
            refs += "\""
            self.graph.csvLines.append(self.createLineTgw(tgw, xcoord, ycoord, refs, self.TGWS_WIDTH_ICON, self.TGWS_HEIGHT_ICON))
            if totalAttachments > 0:
               height     = 40 * totalAttachments
               width      = self.TGWATTACHMENTS_WIDTH
               colsAttch += 1
               if colsAttch > self.TGWATTACHMENTS_COLS:
                  xcoordTableAttachments  = self.TGWATTACHMENTS_XCOORD
                  ycoordTableAttachments  = totalHeightAttachments + self.TGWATTACHMENTS_GAP_ROWS
                  colsAttch = 1
               else:
                  xcoord += (self.TGWATTACHMENTS_WIDTH + TGWATTACHMENTS_GAP_COLS)
               totalHeightAttachments += height 
               lineParent, lineChild = self.createLineTgwTableAttachments(tgw, attachmentsTable, xcoordTableAttachments, ycoordTableAttachments, width, height)
               self.graph.csvLines.append(lineParent)
               self.graph.csvLines.append(lineChild)
            
            cols += 1
            if cols > self.TGWS_COLS:
               xcoord    = self.TGWS_XCOORD
               ycoord   += (self.TGWS_HEIGHT + gapRow)
               cols = 1
            else:
               xcoord += (self.TGWS_WIDTH + gapCol)

    def addVPCs(self, data):
        xcoord = self.VPC_XCOORD
        ycoord = self.VPC_YCOORD
        cols   = 1

        self.graph.vpcs["vpcs"] = []

        #VPCs 
        gapRow = self.VPC_GAP_ROWS
        gapCol = self.VPC_GAP_COLS
        for vpc in data["vpcs"]:
            
            for vpcAz in vpc["azs"]:
                # Store all the existent AZs 
                if vpcAz not in self.graph.azs:
                    self.graph.azs.append(vpcAz)
            
            self.graph.vpcs["vpcs"].append({
                "vpcid": vpc["vpcId"], "xcoord": xcoord, "ycoord": ycoord, "width": self.VPC_WIDTH, "height":  self.VPC_HEIGHT
            })
            self.graph.csvLines.append(self.createLineVpc(vpc, xcoord, ycoord, self.VPC_WIDTH, self.VPC_HEIGHT))
            
            cols += 1
            if cols > self.VPC_COLS:
               xcoord    = self.VPC_XCOORD
               ycoord   += (self.VPC_HEIGHT + gapCol)
               cols = 1
            else:
               xcoord += (self.VPC_WIDTH + gapRow)
        
    def addSubnets(self, data):
        layoutcol = self.SUBNET_COLS
        marginX   = self.SUBNET_XMARGIN
        marginY   = self.SUBNET_YMARGIN
        gapRow    = self.SUBNET_GAP_ROWS
        gapCol    = self.SUBNET_GAP_COLS

        self.graph.subnets["subnets"] = []

        for vpc in data["vpcs"]:
            row = 0
            col = 0

            sortedSubnets = sorted(vpc["subnets"], key=lambda k:(k["az"], k["public"]), reverse=True)
            currentAz     = None
            for subnet in sortedSubnets:
                if not currentAz:
                   currentAz = subnet["az"]

                # Starts another AZ (reset ROW to 0)
                if currentAz != subnet["az"]:
                   currentAz = subnet["az"]
                   row = 0 
                   col = 0

                # Query VPC coordinates
                vpcSubnet = self.findVpc(vpc["vpcId"])
                # Query AZ coordinates
                azVpc     = self.findAzVpc(vpc["vpcId"], subnet["az"])
      
                xcoord = azVpc["xcoord"]
                ycoord = azVpc["ycoord"]
                
                xcoord = (xcoord + marginX) + (self.SUBNET_WIDTH * col)  + ((gapCol * col) if col > 0 else 0)
                ycoord = (ycoord + marginY) + (self.SUBNET_HEIGHT * row) + ((gapRow * row) if row > 0 else 0)
                #print(row,col,xcoord,ycoord,vpc["vpcId"],subnet["az"],subnet["public"])

                self.graph.subnets["subnets"].append({
                    "subnetid": subnet["subnetId"], "xcoord": xcoord, "ycoord": ycoord, "width": self.SUBNET_WIDTH, "height":  self.SUBNET_HEIGHT, "vpcid": vpc["vpcId"]
                })

                self.graph.csvLines.append(self.createLineSubnet(subnet, xcoord, ycoord, subnet["name"], self.SUBNET_WIDTH, self.SUBNET_HEIGHT))
                
                if col >= (layoutcol - 1):
                   col  = 0
                   row += 1
                else:
                   col += 1
        
    def findVpc(self, vpcid):
        for vpc in self.graph.vpcs["vpcs"]:
            if vpcid == vpc["vpcid"]:
               return vpc
        raise ValueError("VPC not found in collection! Something went wrong in your logic, please verify!")
    
    def findAzVpc(self, vpcid, az):
        keyAzByVpc = "{}_{}".format(vpcid,az)
        #for vpc in self.graph.azByVpcs["vpcs"]:
        if keyAzByVpc in self.graph.azByVpcs:
           return self.graph.azByVpcs[keyAzByVpc]
        raise ValueError("AZ by VPC not found in collection! Something went wrong in your logic, please verify!")
    
    def addRouteTables(self, data):
        for vpc in data["vpcs"]:
            sortedSubnets = sorted(vpc["subnets"], key=lambda k:(k["az"], k["public"]))
            for subnet in sortedSubnets:
                subnetInDiagram = self.findRouteTableForSubnet(subnet["subnetId"])
                for rt in subnet["routeTables"]:
                    xcoord = subnetInDiagram["xcoord"] + self.ROUTETABLE_XMARGIN
                    ycoord = subnetInDiagram["ycoord"] + self.ROUTETABLE_YMARGIN
                    self.graph.csvLines.append(self.createLineRouteTable(rt, subnet, xcoord, ycoord, self.ROUTETABLE_WIDTH, self.ROUTETABLE_HEIGHT))

    def addNacls(self, data):
        for vpc in data["vpcs"]:
            sortedSubnets = sorted(vpc["subnets"], key=lambda k:(k["az"], k["public"]))
            for subnet in sortedSubnets:
                subnetInDiagram = self.findRouteTableForSubnet(subnet["subnetId"])
                for nacl in subnet["nacls"]:
                    xcoord = subnetInDiagram["xcoord"] + self.NACLS_XMARGIN
                    ycoord = subnetInDiagram["ycoord"] + self.NACLS_YMARGIN
                    self.graph.csvLines.append(self.createLineNacl(nacl, subnet, xcoord, ycoord, self.NACLS_WIDTH, self.NACLS_HEIGHT))

    def findRouteTableForSubnet(self, subnetid):
        for subnet in self.graph.subnets["subnets"]:
            if subnetid == subnet["subnetid"]:
               return subnet
        raise ValueError("Subnet not found in collection! Something went wrong in your logic, please verify!")

    def addAZs(self, data):
        #AZs
        AZs_Gap_FirstRow = 30
        AZs_Gap_Row      = 5
        for vpc in self.graph.vpcs["vpcs"]:
            idx = 0
            for az in self.graph.azs:
                xcoord = vpc["xcoord"] - int((self.AZ_WIDTH  - self.VPC_WIDTH) / 2)
                ycoord = vpc["ycoord"] + ((idx * (self.AZ_HEIGHT + AZs_Gap_Row)) + AZs_Gap_FirstRow)
                # Gat between AZs
                if idx > 0:
                   ycoord += AZs_Gap_Row
                # else:
                #    ycoord += AZs_Gap_FirstRow

                keyAzByVpcs = "{}_{}".format(vpc["vpcid"],az)
                if not keyAzByVpcs in self.graph.azByVpcs:
                   self.graph.azByVpcs[keyAzByVpcs] = {
                       "xcoord": xcoord,
                       "ycoord": ycoord
                   }

                self.graph.csvLines.append(self.createLineAz(az, vpc, xcoord, ycoord, self.AZ_WIDTH, self.AZ_HEIGHT))
                idx += 1
    
    def createLineAz(self, azid, vpc, x, y, width, height):
        return self.createLine(
                _type="az", _id="{}_{}".format(azid,vpc["vpcid"]), _refs="", 
                _x=x, _y=y, _az=azid, _width=width, _height=height, _tags="az {}".format(azid)
            )
    
    def createLineVpc(self, vpc, x, y, width, height):
        urlVpc = "{}".format(self.buildLink("vpc",vpc["vpcId"]),vpc["vpcId"])
        return self.createLine(
                _type="vpc", _id=vpc["vpcId"], _refs="", _x=x, _y=y, _vpcid=vpc["vpcId"], _cdirblock=vpc["cidrBlock"],
                _name=(vpc["name"] if vpc["name"].rstrip().lstrip() != "" else '" "'),
                _width=width, _height=height,
                _url=urlVpc,
                _tags="vpc"
            )
    
    def createLineSubnet(self, subnet, x, y, name, width, height):
        urlSubnet = "{}".format(self.buildLink("subnet",subnet["subnetId"]),subnet["subnetId"])
        styleSubnet = "publicsubnet" if subnet["public"] else "privatesubnet"
        tags  = "subnet"
        tags += " public" if styleSubnet == "publicsubnet" else " private"
        tags += " " + subnet["az"]
        return self.createLine(
                _type=styleSubnet, _id=subnet["subnetId"], _x=x, _y=y, _cdirblock=subnet["cidrBlock"],
                _width=width, _height=height, _url=urlSubnet, _tags=tags, _name=name
            )
    
    def createLineRouteTable(self, routeTable, subnet, x, y, width, height):
        idRouteTable  = "{}_{}".format(routeTable["routeTableId"],subnet['subnetId'])
        urlRouteTable = "{}".format(self.buildLink("routetable",routeTable["routeTableId"]),routeTable["routeTableId"])
        return self.createLine(
                _type="routetable", _id=idRouteTable, _x=x, _y=y,
                _width=width, _height=height,
                _url=urlRouteTable,
                _resourceidshort = routeTable["routeTableId5Digts"],
                _tags="rt routetable"
            )
    
    def createLineNacl(self, nacl, subnet, x, y, width, height):
        idNacl  = "{}_{}".format(nacl["naclId"],subnet['subnetId'])
        urlNacl = "{}".format(self.buildLink("nacl",nacl["naclId"]),nacl["naclId"])
        return self.createLine(
                _type="nacl", _id=idNacl, _x=x, _y=y,
                _width=width, _height=height,
                _url=urlNacl,
                _resourceidshort = nacl["nacl5Digts"],
                _tags="nacl networkaccesscontrollist"
            )
    
    def createLineTgw(self, tgw, x, y, refs, width, height):
        urlTgw = "{}".format(self.buildLink("transitgateway",tgw["transitGatewayId"]),tgw["transitGatewayId"])
        return self.createLine(
                _type="tgw", _id=tgw["transitGatewayId"], _ownerid=tgw["ownerId"],
                _x=x, _y=y, _refs=refs,
                _width=width, _height=height,
                _url=urlTgw,
                _tags="tgw transitgateway"
            )
    
    def createLineTgwTableAttachments(self, tgw, attachmentTable, x, y, width, height):
        # Parent Container
        parentId = "{}_attachments_container".format(tgw["transitGatewayId"])
        name = "\"&nbsp;<span style='font-size:10px;color:#696969'><i>ATTACHMENTS of </i></span>&nbsp;&nbsp;{}\"".format(tgw["transitGatewayId"])
        lineParent = self.createLine(
                _type="tgw-attachments-container", _id=parentId,
                _x=x, _y=y, _width=width, _height=height,
                _tgwattachmentTable=attachmentTable,
                _name=name,
                _tags="tgwattachments attachments"
            )

        # Child
        childId = "{}_attachments_child".format(tgw["transitGatewayId"])
        lineChild = self.createLine(
                _type="tgw-attachments-table", _id=childId, _ownerid=tgw["ownerId"],
                _resourceparent=parentId,
                _x=x, _y=y, _width=width, _height=height,
                _tgwattachmentTable=attachmentTable,
                _tags="tgwattachments attachments"
            )
        return lineParent, lineChild

    def createLineTgwRouteTable(self, tgwRt, htmlTable, x, y, width, height, refs):
        urlTgwRouteTable = ""
        idTgwRt = tgwRt["routeTableId"]
        return self.createLine(
                _type="tgw-route-table", _id=idTgwRt, _x=x, _y=y,
                _width=width, _height=height, _url=urlTgwRouteTable,
                _tags="tgw-routetable tgw",
                _tgwroutetable=htmlTable,
                _refs=refs
            )

    def createLine(self, **args):
        return "{type},{resourceid},{resourceparent},{refs},{x},{y},{width},{height},{vpcid},{subnetid},{cdirblock},{name},{az},{url},{bottomvpc},{bottomsubnet},{resourceidshort},{ownerid},{tgwattachmenttable},{tags},{tgwroutetable}" \
               .format( \
                type=args["_type"] if "_type" in args else "",
                resourceid=args["_id"] if "_id" in args else "",
                resourceparent=args["_resourceparent"] if "_resourceparent" in args else "\"\"",
                refs=args["_refs"] if "_refs" in args else "",
                x=args["_x"] if "_x" in args else "",
                y=args["_y"] if "_y" in args else "",
                width=args["_width"] if "_width" in args else self.DEFAULT_WIDTH,
                height=args["_height"] if "_height" in args else self.DEFAULT_HEIGHT,
                vpcid=args["_vpcid"] if "_vpcid" in args else "",
                subnetid=args["_subnetid"] if "_subnetid" in args else "",
                cdirblock=args["_cdirblock"] if "_cdirblock" in args else "",
                name=args["_name"] if "_name" in args else "",
                az=args["_az"] if "_az" in args else "",
                url="\"" + args["_url"] + "\"" if "_url" in args else "",
                bottomvpc=args["_bottomvpc"] if "_bottomvpc" in args else self.VPC_BOTTOM,
                bottomsubnet=args["_bottomsubnet"] if "_bottomsubnet" in args else self.SUBNET_BOTTOM,
                resourceidshort=args["_resourceidshort"] if "_resourceidshort" in args else "",
                ownerid=args["_ownerid"] if "_ownerid" in args else "",
                tgwattachmenttable=args["_tgwattachmentTable"] if "_tgwattachmentTable" in args else "",
                tags="\"" + args["_tags"] + "\"" if "_tags" in args else "",
                tgwroutetable=args["_tgwroutetable"] if "_tgwroutetable" in args else ""
            )

class Graph:

    def __init__(self):
        self.csvLines = []
        self.azs      = []
        self.azByVpcs = {}
        self.vpcs     = {}
        self.subnets  = {}





import webbrowser
import os, sys
import threading


class PymxGraphTree:

    def __init__(self, pathToResources, images, htmlSnippets):
        self.pathToResources = pathToResources
        self.images          = images
        self.htmlSnippets    = htmlSnippets

    def openHTML(self,html):
        directory = os.path.dirname(os.path.realpath(__file__))
        fileName  = os.path.join(str(directory),"tree.html")
        try:
            os.remove(fileName)
        except OSError:
            pass

        with open(fileName, "w") as file:
            file.write(html)
        webbrowser.open('file://' + os.path.realpath(fileName))    


    def drawTree(self, objTree):
        html = """
        <html>
         <head>
            """ + self.htmlSnippets.htmlTitle('Tree') + """
            """ + self.htmlSnippets.jsLibraries()        + """
            <script type="text/javascript">
                function TreeNodeShape() { };
        
                TreeNodeShape.prototype = new mxCylinder();
                TreeNodeShape.prototype.constructor = TreeNodeShape;
        
                // Defines the length of the upper edge segment.
                TreeNodeShape.prototype.segment = 20;
        
                // Needs access to the cell state for rendering
                TreeNodeShape.prototype.apply = function(state)
                {
                    mxCylinder.prototype.apply.apply(this, arguments);
                    this.state = state;
                };
                
                TreeNodeShape.prototype.redrawPath = function(path, x, y, w, h, isForeground)
                {
                    var graph = this.state.view.graph;
                    var hasChildren = graph.model.getOutgoingEdges(this.state.cell).length > 0;
                    
                    if (isForeground)
                    {
                        if (hasChildren)
                        {
                            // Painting outside of vertex bounds is used here
                            path.moveTo(w / 2, h + this.segment);
                            path.lineTo(w / 2, h);
                            path.end();
                        }
                    }
                    else
                    {
                        path.moveTo(0, 0);
                        path.lineTo(w, 0);
                        path.lineTo(w, h);
                        path.lineTo(0, h);
                        path.close();
                    }
                };
                
                mxCellRenderer.registerShape('treenode', TreeNodeShape);
        
                // Defines a custom perimeter for the nodes in the tree
                mxGraphView.prototype.updateFloatingTerminalPoint = function(edge, start, end, source)
                {
                    var pt = null;
                    
                    if (source)
                    {
                        pt = new mxPoint(start.x + start.width / 2,
                                start.y + start.height + TreeNodeShape.prototype.segment);
                    }
                    else
                    {
                        pt = new mxPoint(start.x + start.width / 2, start.y);
                    }
        
                    edge.setAbsoluteTerminalPoint(pt, source);
                };
            </script>
    
            <script type="text/javascript">
                function main()
                {
                    """ + self.htmlSnippets.jsBrowserNotSupported() + """
                    else
                    {
                        mxGraph.prototype.collapsedImage = new mxImage('""" + self.images.collapsedImagePath + """', 9, 9);
                        mxGraph.prototype.expandedImage = new mxImage('""" + self.images.expandedImagePath + """', 9, 9);

                        var container = document.createElement('div');
                        container.style.position = 'absolute';
                        container.style.overflow = 'hidden';
                        container.style.left = '0px';
                        container.style.top = '0px';
                        container.style.right = '0px';
                        container.style.bottom = '0px';
                        
                        if (mxClient.IS_IE) {
                            new mxDivResizer(container);
                        }

                        document.body.appendChild(container);

                        var graph = new mxGraph(container);

                        graph.keepEdgesInBackground = true;

                        var style = graph.getStylesheet().getDefaultVertexStyle();
                        style[mxConstants.STYLE_SHAPE] = 'treenode';
                        style[mxConstants.STYLE_GRADIENTCOLOR] = 'white';
                        style[mxConstants.STYLE_SHADOW] = true;
                        
                        style = graph.getStylesheet().getDefaultEdgeStyle();
                        style[mxConstants.STYLE_EDGE] = mxEdgeStyle.TopToBottom;
                        style[mxConstants.STYLE_ROUNDED] = true;

                        graph.setAutoSizeCells(true);
                        graph.setPanning(true);
                        graph.panningHandler.useLeftButtonForPanning = true;

                        var keyHandler = new mxKeyHandler(graph);
                        var layout = new mxCompactTreeLayout(graph, false);
                        layout.useBoundingBox = false;
                        layout.edgeRouting = false;
                        layout.levelDistance = 30;
                        layout.nodeDistance = 10;

                        // To allow wrapping
                        graph.setHtmlLabels(true);

                        var layoutMgr = new mxLayoutManager(graph);

                        layoutMgr.getLayout = function(cell) {
                            if (cell.getChildCount() > 0)
                            {
                                return layout;
                            }
                        };

                        graph.setCellsSelectable(false);

                        graph.isCellFoldable = function(cell) {
                            return this.model.getOutgoingEdges(cell).length > 0;
                        };
        
                        // Defines the position of the folding icon
                        graph.cellRenderer.getControlBounds = function(state) {
                            if (state.control != null) {
                                var oldScale = state.control.scale;
                                var w = state.control.bounds.width / oldScale;
                                var h = state.control.bounds.height / oldScale;
                                var s = state.view.scale;			
        
                                return new mxRectangle(state.x + state.width / 2 - w / 2 * s,
                                    state.y + state.height + TreeNodeShape.prototype.segment * s - h / 2 * s,
                                    w * s, h * s);
                            }
                            
                            return null;
                        };

                        // Implements the click on a folding icon
                        graph.foldCells = function(collapse, recurse, cells)
                        {
                            this.model.beginUpdate();
                            try
                            {
                                toggleSubtree(this, cells[0], !collapse);
                                this.model.setCollapsed(cells[0], collapse);
        
                                // Executes the layout for the new graph since
                                // changes to visiblity and collapsed state do
                                // not trigger a layout in the current manager.
                                layout.execute(graph.getDefaultParent());
                            }
                            finally
                            {
                                this.model.endUpdate();
                            }
                        };

                        
                        """ +  self.htmlSnippets.jsMxToolBox() + """

                        // Gets the default parent for inserting new cells. This
                        // is normally the first child of the root (ie. layer 0).
                        var parent = graph.getDefaultParent();
                                        
                        // Adds the root vertex of the tree
                        graph.getModel().beginUpdate();
                        try
                        {
                            """ + objTree + """
                            layout.execute(graph.getDefaultParent());
                        }
                        finally
                        {
                            // Updates the display
                            graph.getModel().endUpdate();
                        }
                    }
                };
                // Updates the visible state of a given subtree taking into
                // account the collapsed state of the traversed branches
                function toggleSubtree(graph, cell, show)
                {
                    show = (show != null) ? show : true;
                    var cells = [];
                    
                    graph.traverse(cell, true, function(vertex)
                    {
                        if (vertex != cell)
                        {
                            cells.push(vertex);
                        }
        
                        // Stops recursion if a collapsed cell is seen
                        return vertex == cell || !graph.isCellCollapsed(vertex);
                    });
        
                    graph.toggleCells(show, cells, true);
                };

                """ + self.htmlSnippets.jsFormatXml() + """
            </script>
         </head>
            <body onload="main();">
            </body>
            """ + self.htmlSnippets.htmlFormDrawIoEdit() + """
        </html>
        """
        self.openHTML(html)    
